import { __decorate } from 'tslib';
import { ɵɵdefineInjectable, Injectable, Component, NgModule, ɵɵinject } from '@angular/core';
import { BehaviorSubject, Subject } from 'rxjs';
import { isNullOrUndefined, Toolkit2, Notify, NotifyConfig, PairKeyValue, RestServiceResultPolicy, RestServiceErrorPolicy } from '@angular-boot/util';
import { CommonModule } from '@angular/common';
import swal from 'sweetalert2';
import { HttpHeaders, HttpClient } from '@angular/common/http';
import { map, takeUntil } from 'rxjs/operators';

let CoreService = class CoreService {
    constructor() { }
};
CoreService.ngInjectableDef = ɵɵdefineInjectable({ factory: function CoreService_Factory() { return new CoreService(); }, token: CoreService, providedIn: "root" });
CoreService = __decorate([
    Injectable({
        providedIn: 'root'
    })
], CoreService);

let CoreComponent = class CoreComponent {
    constructor() { }
    ngOnInit() {
    }
};
CoreComponent = __decorate([
    Component({
        selector: 'lib-core',
        template: `
    <p>
      core works!
    </p>
  `
    })
], CoreComponent);

let CoreModule = class CoreModule {
};
CoreModule = __decorate([
    NgModule({
        declarations: [CoreComponent],
        imports: [],
        exports: [CoreComponent]
    })
], CoreModule);

class StoredItem {
    constructor(key) {
        this.value = new BehaviorSubject(null);
        this.exists = new BehaviorSubject(true);
        this.currentValue = this.value.asObservable();
        this.currentExists = this.exists.asObservable();
        this.key = key;
    }
    changeValue(value) {
        this.value.next(value);
    }
    changExists(value) {
        this.exists.next(value);
    }
}

/**
 * Abstract cache storage
 */
class CacheStorageAbstract {
}

/**
 * Service for storing data in session storage
 */
let CacheSessionStorage = class CacheSessionStorage extends CacheStorageAbstract {
    getItem(key) {
        let value = sessionStorage.getItem(key);
        return value ? JSON.parse(value) : null;
    }
    setItem(key, value) {
        try {
            sessionStorage.setItem(key, JSON.stringify(value));
            return true;
        }
        catch (e) {
            return false;
        }
    }
    removeItem(key) {
        sessionStorage.removeItem(key);
    }
    clear() {
        sessionStorage.clear();
    }
    type() {
        return 1 /* SESSION_STORAGE */;
    }
    isEnabled() {
        try {
            const tmp = new Date().getTime();
            sessionStorage.setItem('test' + tmp, 'test');
            sessionStorage.removeItem('test' + tmp);
            return true;
        }
        catch (e) {
            return false;
        }
    }
};
CacheSessionStorage.ngInjectableDef = ɵɵdefineInjectable({ factory: function CacheSessionStorage_Factory() { return new CacheSessionStorage(); }, token: CacheSessionStorage, providedIn: "root" });
CacheSessionStorage = __decorate([
    Injectable({
        providedIn: 'root'
    })
], CacheSessionStorage);

/**
 * Service for storing data in local storage
 */
let CacheLocalStorage = class CacheLocalStorage extends CacheStorageAbstract {
    getItem(key) {
        let value = localStorage.getItem(key);
        return value ? JSON.parse(value) : null;
    }
    setItem(key, value) {
        try {
            localStorage.setItem(key, JSON.stringify(value));
            return true;
        }
        catch (e) {
            return false;
        }
    }
    removeItem(key) {
        localStorage.removeItem(key);
    }
    clear() {
        localStorage.clear();
    }
    type() {
        return 0 /* LOCAL_STORAGE */;
    }
    isEnabled() {
        try {
            const tmp = new Date().getTime();
            localStorage.setItem('test' + tmp, 'test');
            localStorage.removeItem('test' + tmp);
            return true;
        }
        catch (e) {
            return false;
        }
    }
};
CacheLocalStorage.ngInjectableDef = ɵɵdefineInjectable({ factory: function CacheLocalStorage_Factory() { return new CacheLocalStorage(); }, token: CacheLocalStorage, providedIn: "root" });
CacheLocalStorage = __decorate([
    Injectable({
        providedIn: 'root'
    })
], CacheLocalStorage);

/**
 * Service for storing data in local storage
 */
let CacheMemoryStorage = class CacheMemoryStorage extends CacheStorageAbstract {
    /**
     * Service for storing data in local storage
     */
    constructor() {
        super(...arguments);
        this._data = {};
    }
    getItem(key) {
        return this._data[key] ? this._data[key] : null;
    }
    setItem(key, value) {
        this._data[key] = value;
        return true;
    }
    removeItem(key) {
        delete this._data[key];
    }
    clear() {
        this._data = [];
    }
    type() {
        return 2 /* MEMORY */;
    }
    isEnabled() {
        return true;
    }
};
CacheMemoryStorage.ngInjectableDef = ɵɵdefineInjectable({ factory: function CacheMemoryStorage_Factory() { return new CacheMemoryStorage(); }, token: CacheMemoryStorage, providedIn: "root" });
CacheMemoryStorage = __decorate([
    Injectable({
        providedIn: 'root'
    })
], CacheMemoryStorage);

// const CACHE_PREFIX = 'CacheService';
const CACHE_PREFIX = '';
let CacheService = class CacheService {
    constructor(localStorage, sessionStorage, memoryStorage) {
        this.localStorage = localStorage;
        this.sessionStorage = sessionStorage;
        this.memoryStorage = memoryStorage;
        /**
         * Cache prefix
         */
        this._prefix = CACHE_PREFIX;
        /**
         * Default cache options
         * CacheOptionsInterface
         */
        this._defaultOptions = {
            expires: Number.MAX_VALUE,
            maxAge: Number.MAX_VALUE
        };
        this.localStorageStoredItemList = [];
        this.sessionStorageStoredItemList = [];
        this.memoryStoredItemList = [];
    }
    // Start
    setItem(key, value, storageType, options) {
        this.set(key, value, storageType, options);
        this.setToStoredList(key, value, storageType, true);
    }
    getItem(key, storageType) {
        const val = this.get(key, storageType);
        if (val !== null) {
            if (isNullOrUndefined(this.getRelatedList(storageType).find(e => e.key === key))) {
                this.setToStoredList(key, val, storageType, true);
            }
            return this.getRelatedList(storageType).find(e => e.key === key).currentValue;
        }
        this.set(key, null, storageType);
        this.setToStoredList(key, null, storageType, false);
        return this.getRelatedList(storageType).find(e => e.key === key).currentValue;
    }
    existsItem(key, storageType) {
        const val = this.get(key, storageType);
        if (val !== null) {
            if (isNullOrUndefined(this.getRelatedList(storageType).find(e => e.key === key))) {
                this.setToStoredList(key, val, storageType, true);
            }
            return this.getRelatedList(storageType).find(e => e.key === key).currentExists;
        }
        if (this.getRelatedList(storageType).find(e => e.key === key) === undefined) {
            return new BehaviorSubject(false);
        }
        return this.getRelatedList(storageType).find(e => e.key === key).currentExists;
    }
    removeItem(key, storageType) {
        return this.remove(key, storageType);
    }
    // End
    setToStoredList(key, value, storageType, exists) {
        this.getRelatedList(storageType).push(new StoredItem(key));
        this.getRelatedList(storageType).find(e => e.key === key).changeValue(value);
        this.getRelatedList(storageType).find(e => e.key === key).changExists(exists);
    }
    getRelatedList(storageType) {
        switch (storageType) {
            case 0 /* LOCAL_STORAGE */:
                return this.localStorageStoredItemList;
                break;
            case 1 /* SESSION_STORAGE */:
                return this.sessionStorageStoredItemList;
                break;
            case 2 /* MEMORY */:
                return this.memoryStoredItemList;
                break;
        }
    }
    /**
     * Get data from cache
     * key
     * {any}
     */
    get(key, storageType) {
        const storageValue = this.getMyValidatedStorage(storageType).getItem(this._toStorageKey(key));
        let value = null;
        if (storageValue) {
            if (this._validateStorageValue(storageValue)) {
                value = storageValue.value;
            }
            else {
                this.remove(key, storageType);
            }
        }
        return value;
    }
    set(key, value, storageType, options) {
        const storageKey = this._toStorageKey(key);
        options = options ? options : this._defaultOptions;
        if (this.getMyValidatedStorage(storageType)
            .setItem(storageKey, this._toStorageValue(value, options))) {
            if (!this._isSystemKey(key) && options.tag) {
                this._saveTag(options.tag, storageKey, storageType);
            }
            return true;
        }
        return false;
    }
    /**
     * Check if value exists
     * key
     * {boolean}
     */
    exists(key, storageType) {
        return !!this.get(key, storageType);
    }
    /**
     * Remove item from cache
     * key
     */
    remove(key, storageType) {
        this.getMyValidatedStorage(storageType).removeItem(this._toStorageKey(key));
        this._removeFromTag(this._toStorageKey(key), storageType);
        this.removeFromStoredList(key, storageType);
    }
    /**
     * Remove key from tags keys list
     */
    _removeFromTag(key, storageType) {
        const tags = this.get(this._tagsStorageKey(), storageType) || {};
        let index;
        for (let tag in tags) {
            index = tags[tag].indexOf(key);
            if (index !== -1) {
                tags[tag].splice(index, 1);
                this.set(this._tagsStorageKey(), tags, storageType);
                break;
            }
        }
    }
    /**
     * Remove all from cache
     */
    removeAll(storageType) {
        this.getMyValidatedStorage(storageType).clear();
    }
    /**
     * Get all tag data
     */
    getTagData(tag, storageType) {
        let tags = this.get(this._tagsStorageKey(), storageType) || {};
        let result = {};
        if (tags[tag]) {
            tags[tag].forEach((key) => {
                let data = this.get(this._fromStorageKey(key), storageType);
                if (data) {
                    result[this._fromStorageKey(key)] = data;
                }
            });
        }
        return result;
    }
    /**
     * Remove all by tag
     */
    removeTag(tag, storageType) {
        storageType = storageType ? storageType : this._defaultStorageType;
        let tags = this.get(this._tagsStorageKey(), storageType) || {};
        if (tags[tag]) {
            tags[tag].forEach((key) => {
                this.getMyValidatedStorage(storageType).removeItem(key);
            });
            delete tags[tag];
            this.set(this._tagsStorageKey(), tags, storageType);
        }
    }
    /**
     * Set global cache key prefix
     */
    setGlobalPrefix(prefix) {
        this._prefix = prefix;
    }
    _toStorageKey(key) {
        return this._getCachePrefix() + key;
    }
    _fromStorageKey(key) {
        return key.replace(this._getCachePrefix(), '');
    }
    _getCachePrefix() {
        return this._prefix;
    }
    getMyValidatedStorage(storageType) {
        switch (storageType) {
            case 2 /* MEMORY */:
                return this._validateStorage(this.memoryStorage);
                // return this.memoryStorage;
                break;
            case 0 /* LOCAL_STORAGE */:
                return this._validateStorage(this.localStorage);
                // return this.localStorage;
                break;
            case 1 /* SESSION_STORAGE */:
                return this._validateStorage(this.sessionStorage);
                // return this.sessionStorage;
                break;
        }
    }
    /**
     * Validate cache storage
     */
    _validateStorage(cacheStorage) {
        if (!cacheStorage.isEnabled()) {
            return this.memoryStorage;
        }
        return cacheStorage;
    }
    /**
     * Validate storage value
     */
    _validateStorageValue(value) {
        return !!value.options.expires && value.options.expires > Date.now();
    }
    /**
     * Prepare value to set to storage
     */
    _toStorageValue(value, options) {
        return {
            value: value,
            options: this._toStorageOptions(options)
        };
    }
    /**
     * Prepare options to set to storage
     */
    _toStorageOptions(options) {
        const storageOptions = {};
        storageOptions.expires = options.expires ? options.expires :
            (options.maxAge ? Date.now() + (options.maxAge * 1000) : this._defaultOptions.expires);
        storageOptions.maxAge = options.maxAge ? options.maxAge : this._defaultOptions.maxAge;
        return storageOptions;
    }
    /**
     * Save tag to list of tags
     */
    _saveTag(tag, key, storageType) {
        const tags = this.get(this._tagsStorageKey(), storageType) || {};
        if (!tags[tag]) {
            tags[tag] = [key];
        }
        else {
            tags[tag].push(key);
        }
        this.set(this._tagsStorageKey(), tags, storageType);
    }
    _tagsStorageKey() {
        // return 'CacheService_tags';
        return '';
    }
    /**
     * check if its system cache key
     */
    _isSystemKey(key) {
        return [this._tagsStorageKey()].indexOf(key) !== -1;
    }
    removeFromStoredList(key, storageType) {
        this.getRelatedList(storageType).find(e => e.key === key).changeValue(null);
        this.getRelatedList(storageType).find(e => e.key === key).changExists(false);
        this.getRelatedList(storageType).filter(e => {
            return e.key !== key;
        });
    }
};
CacheService.ctorParameters = () => [
    { type: CacheLocalStorage },
    { type: CacheSessionStorage },
    { type: CacheMemoryStorage }
];
CacheService.ngInjectableDef = ɵɵdefineInjectable({ factory: function CacheService_Factory() { return new CacheService(ɵɵinject(CacheLocalStorage), ɵɵinject(CacheSessionStorage), ɵɵinject(CacheMemoryStorage)); }, token: CacheService, providedIn: "root" });
CacheService = __decorate([
    Injectable({
        providedIn: 'root'
    })
], CacheService);

let CacheModule = class CacheModule {
};
CacheModule = __decorate([
    NgModule({
        declarations: [],
        imports: [
            CommonModule
        ],
        providers: [
            CacheService
        ]
    })
], CacheModule);

// export abstract class ServiceConfig {
//    abstract getUrl(): string;
//    abstract getToken(): string;
// }
class ServiceConfig {
}

/**
 * @author Jafar Amini in March 2018.
 */
var TokenMode;
(function (TokenMode) {
    TokenMode[TokenMode["URL"] = 'URL'] = "URL";
    TokenMode[TokenMode["HEADER"] = 'HEADER'] = "HEADER";
})(TokenMode || (TokenMode = {}));

class GlobalConfigurations {
}

/**
 * @author Jafar Amini in March 2018.
 */
class ErrorHandle {
    static handleDefault(error, _ServiceConfig) {
        console.error(error);
        switch (error.status) {
            case 0:
                Notify.showNotify(new NotifyConfig(Notify.Type.WARNING, Notify.Placement.TOP_LEFT, Notify.TEMPLATES.Template2, 'خطا در برقراری ارتباط با سرور ', 'خطا'));
                break;
            case 400:
                Notify.showNotify(new NotifyConfig(Notify.Type.WARNING, Notify.Placement.TOP_LEFT, Notify.TEMPLATES.Template2, 'اطلاعات ورودی درست نیست', 'خطا'));
                break;
            case 403:
                Notify.showNotify(new NotifyConfig(Notify.Type.WARNING, Notify.Placement.TOP_LEFT, Notify.TEMPLATES.Template2, 'درخواست غیرمجاز', 'توجه'));
                break;
            case 401:
                Notify.showNotify(new NotifyConfig(Notify.Type.WARNING, Notify.Placement.TOP_LEFT, Notify.TEMPLATES.Template2, 'نام کاربری یا رمز اشتباه است. یا اکانت فعال نشده است', 'توجه'));
                _ServiceConfig.redirectToLoginPage();
                break;
            case 404:
                Notify.showNotify(new NotifyConfig(Notify.Type.WARNING, Notify.Placement.TOP_LEFT, Notify.TEMPLATES.Template2, 'درخواست مورد نظر پیدا نشد', 'توجه'));
                break;
            default:
                const notifyConfig = new Toolkit2.NotifyConfig(Toolkit2.Notify.Type.DANGER, Toolkit2.Notify.Placement.TOP_LEFT, 
                // null,
                Toolkit2.Notify.TEMPLATES.Template2, 'خطایی رخ داد. کد خطا: '
                    + error.status, 'توجه شود');
                // notifyConfig.options.icon = 'fa fa-eye fa-2x';
                Toolkit2.Notify.showNotify(notifyConfig);
        }
    }
    static alertMessage(messages) {
        swal({
            showCloseButton: true,
            confirmButtonColor: '#04383c',
            background: '#e4e4e4',
            type: 'error',
            title: 'خطا!',
            html: '<p>' + messages + '</p>',
            confirmButtonText: 'تایید'
        });
    }
}

/**
 * @author Jafar Amini in March 2018.
 */
class Config {
    static getFakeUrl() {
        return Config._fakeUrl;
    }
    static setFakeUrl(fakeUrl) {
        Config._fakeUrl = fakeUrl;
    }
}
// public static getFakeUrl() {
//   // return this._fakeUrl;
//   return '/api/';
// }
Config._fakeUrl = '/api/';

/**
 * @author Jafar Amini in March 2018.
 */
class FakeData {
    constructor() { }
    static getDomain(domainName) {
        let t = JSON.parse(localStorage.getItem(domainName));
        if (isNullOrUndefined(t)) {
            t = new Array();
        }
        return t;
    }
}

/**
 * @author Jafar Amini in March 2018.
 */
// import {LocalStorage} from '../../../app/configuration/LocalStorage';
// @Injectable()
class FakeServiceBase {
    constructor() {
        this._prefix = '';
    }
    getService(_objectsuffix) {
        let headers = new HttpHeaders();
        // headers =  headers.set('Authorization', LocalStorage.getEncodedBasicAuth());
        return this._HttpClient.get(this.url + this._prefix + this._objectName + '/'
            + _objectsuffix, { headers: headers });
    }
    postService(value, _objectsuffix) {
        // console.log('finalUrl: ',this.url + this._prefix + this._objectName + '/' + _objectsuffix);
        let headers = new HttpHeaders();
        headers = headers
            .set('Content-Type', 'application/json');
        return this._HttpClient.post(this.url + this._prefix + this._objectName + '/' +
            _objectsuffix, value, { headers: headers });
    }
    putService(value, _objectsuffix) {
        const headers = new HttpHeaders({ 'Content-Type': 'application/json'
            // , 'Authorization': LocalStorage.getEncodedBasicAuth()
        });
        return this._HttpClient.put(this.url + this._prefix + this._objectName + '/' +
            _objectsuffix
        // + '?Authorization=' + Config.getLocalStorageToken()
        , JSON.stringify(value), { headers: headers });
    }
    deleteService(id) {
        let headers = new HttpHeaders();
        // headers = headers.set('Authorization', LocalStorage.getEncodedBasicAuth());
        return this._HttpClient.delete(this.url + this._prefix + this._objectName + '/' + (id), { headers: headers });
    }
}

/**
 * @author Jafar Amini in March 2018.
 */
var Keyword;
(function (Keyword) {
    Keyword[Keyword["OK"] = 'OK'] = "OK";
    Keyword[Keyword["DELETED"] = 'با موفقیت حذف شد '] = "DELETED";
    Keyword[Keyword["NOT_FOUND"] = ' موردی یافت نشد'] = "NOT_FOUND";
    Keyword[Keyword["FAILURE"] = 'مشکلی رخ داد'] = "FAILURE";
    Keyword[Keyword["NOT_DONE"] = 'انجام نشد'] = "NOT_DONE";
    Keyword[Keyword["USER_NAME_DUPLICATE"] = 'نام کاربری تکراری است'] = "USER_NAME_DUPLICATE";
})(Keyword || (Keyword = {}));

/**
 * @author Jafar Amini in March 2018.
 */
class HandleResponse {
    static getMessagesAsString(states, separator) {
        let totalMessage = '';
        for (const state of states) {
            if (state.message !== '') {
                totalMessage = totalMessage + separator + state.message;
            }
            else {
                totalMessage = totalMessage + separator + Keyword[state.keyword];
            }
        }
        return totalMessage;
    }
    static getMessagesAsArray(states) {
        const totalMessage = new Array();
        for (const state of states) {
            if (state.message !== '') {
                totalMessage.push(state.message);
            }
            else {
                totalMessage.push(Keyword[state.keyword]);
            }
        }
        return totalMessage;
    }
}

class ResponseContent {
}

class State {
}

/**
 * @author Jafar Amini in March 2018.
 */
// @Injectable()
class FakeServiceBase2 extends FakeServiceBase {
    constructor() {
        super();
        this.url = Config.getFakeUrl();
    }
    getService(_url) {
        const ret = new Subject();
        super.getService(_url).subscribe((res) => {
            // console.log(_url + '------->' , res )
            // ;
            if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
                ret.next(res);
            }
            else if (res.flag) {
                ret.next(res.data);
            }
            else if (!res.flag) {
                // console.log('operation failed!');
                const from = 'top';
                const align = 'center';
                const type_ = 'danger';
                const msg = '';
                // console.log('عملیات انجام نشد');
                for (const state of res.states) {
                    if (state.message !== '') {
                        // Notifyme.showNotify(from, align, type_, state.message);
                        // alert(state.message);
                        swal({
                            type: 'warning',
                            title: state.message,
                            confirmButtonText: 'تایید!'
                        });
                    }
                    else {
                        // Notifyme.showNotify(from, align, type_, Keyword[state.keyword]);
                        // alert(Keyword[state.keyword]);
                        swal({
                            type: 'warning',
                            title: Keyword[state.keyword],
                            confirmButtonText: 'تایید!'
                        });
                    }
                }
            }
        }, error => {
            // if (error.status === 500){
            //   alert(500);
            //   alert(error._body)
            // }
            if (error.status === 403) {
                // alert("درخواست غیرمجاز");
                swal({
                    type: 'warning',
                    title: 'درخواست غیرمجاز',
                    confirmButtonText: 'تایید!'
                });
            }
            else {
                alert('error.status ' + error.status);
                // console.log('error---> ' + error);
                // console.log('error._body ' + error._body)
            }
        });
        return ret.asObservable();
    }
    getService_f(_url) {
        const ret = new Subject();
        super.getService(_url).subscribe((res) => {
            // console.log(_url + '------->' , res );
            if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
                ret.next(res);
            }
            else if (res.flag) {
                ret.next(res.data);
            }
            else if (!res.flag) {
                ret.error(res);
            }
        }, error => {
            // if (error.status === 500){
            //   alert(500);
            //   alert(error._body)
            // }
            if (error.status === 403) {
                swal({
                    type: 'warning',
                    title: 'درخواست غیرمجاز',
                    confirmButtonText: 'تایید!'
                });
            }
            else {
                alert('error.status ' + error.status);
                // console.log('error---> ' + error);
                // console.log('error._body ' + error._body)
            }
        });
        return ret.asObservable();
    }
    updateFakeDatabase(_objectName, objectData) {
        localStorage.setItem(_objectName, JSON.stringify(objectData));
    }
    postService(value, _url) {
        // console.log('value: ', value);
        const ret = new Subject();
        super.postService(value, _url).subscribe((res) => {
            super.getService('').subscribe((res2) => {
                // console.log('ressssssssss: ', res2);
                this.updateFakeDatabase(this._objectName, res2);
            });
            // console.log('response =======>', res);
            if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
                ret.next(res);
            }
            else if (res.flag) {
                ret.next(res.data);
            }
            else if (!res.flag) {
                // console.log('operation failed!');
                const from = 'top';
                const align = 'center';
                const type_ = 'danger';
                const msg = '';
                for (const state of res.states) {
                    if (state.message !== '') {
                        // Notifyme.showNotify(from, align, type_, state.message);
                        // alert(state.message);
                        swal({
                            showCloseButton: true,
                            confirmButtonColor: '#04383c',
                            background: '#e4e4e4',
                            type: 'error',
                            title: 'خطا!',
                            html: '<p>' + state.message + '</p>',
                            confirmButtonText: 'تایید'
                        });
                    }
                    else {
                        // Notifyme.showNotify(from, align, type_, Keyword[state.keyword]);
                        // alert(Keyword[state.keyword]);
                        swal({
                            showCloseButton: true,
                            confirmButtonColor: '#04383c',
                            background: '#e4e4e4',
                            type: 'error',
                            title: 'خطا!',
                            html: '<p>' + Keyword[state.keyword] + '</p>',
                            confirmButtonText: 'تایید'
                        });
                    }
                }
            }
        }, error => {
            // if (error.status === 500){
            //   alert(500);
            //   alert(error._body)
            // }
            if (error.status === 403) {
                // alert("درخواست غیرمجاز");
                swal({
                    type: 'warning',
                    title: 'درخواست غیرمجاز',
                    confirmButtonText: 'تایید!'
                });
            }
            else {
                alert('error.status ' + error.status);
                // console.log('error---> ' + error);
                // console.log('error._body ' + error._body)
            }
        });
        return ret.asObservable();
    }
    postService_f(value, _url) {
        const ret = new Subject();
        super.postService(value, _url).subscribe((res) => {
            // console.log('response =======>', res);
            if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
                ret.next(res);
            }
            else if (res.flag) {
                ret.next(res.data);
            }
            else if (!res.flag) {
                ret.error(res);
            }
        }, error => {
            // if (error.status === 500){
            //   alert(500);
            //   alert(error._body)
            // }
            if (error.status === 403) {
                swal({
                    type: 'warning',
                    title: 'درخواست غیرمجاز',
                    confirmButtonText: 'تایید!'
                });
            }
            else {
                alert('error.status ' + error.status);
                // console.log('error---> ' + error);
                // console.log('error._body ' + error._body)
            }
        });
        return ret.asObservable();
    }
    putService(value, _url) {
        const ret = new Subject();
        super.putService(value, _url).subscribe((res) => {
            // console.log('response =======>', res);
            if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
                ret.next(res);
            }
            else if (res.flag) {
                ret.next(res.data);
            }
            else if (!res.flag) {
                // console.log('operation failed!');
                const from = 'top';
                const align = 'center';
                const type_ = 'danger';
                const msg = '';
                for (const state of res.states) {
                    if (state.message !== '') {
                        // Notifyme.showNotify(from, align, type_, state.message);
                        // alert(state.message);
                        swal({
                            type: 'warning',
                            title: state.message,
                            confirmButtonText: 'تایید!'
                        });
                    }
                    else {
                        // Notifyme.showNotify(from, align, type_, Keyword[state.keyword]);
                        // alert(Keyword[state.keyword]);
                        swal({
                            type: 'warning',
                            title: Keyword[state.keyword],
                            confirmButtonText: 'تایید!'
                        });
                    }
                }
            }
        }, error => {
            // if (error.status === 500){
            //   alert(500);
            //   alert(error._body)
            // }
            if (error.status === 403) {
                // alert("درخواست غیرمجاز");
                swal({
                    type: 'warning',
                    title: 'درخواست غیرمجاز',
                    confirmButtonText: 'تایید!'
                });
            }
            else {
                alert('error.status ' + error.status);
                // console.log('error---> ' + error);
                // console.log('error._body ' + error._body)
            }
        });
        return ret.asObservable();
    }
    putService_f(value, _url) {
        const ret = new Subject();
        super.putService(value, _url).subscribe((res) => {
            // console.log('response =======>', res);
            if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
                ret.next(res);
            }
            else if (res.flag) {
                ret.next(res.data);
            }
            else if (!res.flag) {
                ret.error(res);
            }
        }, error => {
            // if (error.status === 500){
            //   alert(500);
            //   alert(error._body)
            // }
            if (error.status === 403) {
                // alert("درخواست غیرمجاز");
                swal({
                    type: 'warning',
                    title: 'درخواست غیرمجاز',
                    confirmButtonText: 'تایید!'
                });
            }
            else {
                alert('error.status ' + error.status);
            }
            // console.log('error---> ' + error);
            // console.log('error._body ' + error._body)
        });
        return ret.asObservable();
    }
    deleteService(parentId) {
        const ret = new Subject();
        super.deleteService(parentId).subscribe((res) => {
            // console.log('res---------------...>>', res);
            super.getService('').subscribe((res2) => {
                // console.log('ressssssssss: ', res2);
                this.updateFakeDatabase(this._objectName, res2);
                if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
                    ret.next(res);
                }
                else if (res.flag) {
                    ret.next(res.data);
                }
                else if (!res.flag) {
                    // console.log('operation failed!');
                    const from = 'top';
                    const align = 'center';
                    const type_ = 'danger';
                    const msg = '';
                    for (const state of res.states) {
                        if (state.message !== '') {
                            // Notifyme.showNotify(from, align, type_, state.message);
                            // alert(state.message);
                            swal({
                                type: 'warning',
                                title: state.message,
                                confirmButtonText: 'تایید!'
                            });
                        }
                        else {
                            // Notifyme.showNotify(from, align, type_, Keyword[state.keyword]);
                            // alert(Keyword[state.keyword]);
                            swal({
                                type: 'warning',
                                title: Keyword[state.keyword],
                                confirmButtonText: 'تایید!'
                            });
                        }
                    }
                }
            });
        }, error => {
            // if (error.status === 500){
            //   alert(500);
            //   alert(error._body)
            // }
            if (error.status === 403) {
                // alert("درخواست غیرمجاز");
                swal({
                    type: 'warning',
                    title: 'درخواست غیرمجاز',
                    confirmButtonText: 'تایید!'
                });
            }
            else {
                alert('error.status ' + error.status);
            }
            // console.log('error---> ' + error);
            // console.log('error._body ' + error._body)
        });
        return ret.asObservable();
    }
    deleteService_f(parentId) {
        const ret = new Subject();
        super.deleteService(parentId).subscribe((res) => {
            if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
                ret.next(res);
            }
            else if (res.flag) {
                ret.next(res.data);
            }
            else if (!res.flag) {
                ret.error(res);
            }
        }, error => {
            // if (error.status === 500){
            //   alert(500);
            //   alert(error._body)
            // }
            if (error.status === 403) {
                // alert("درخواست غیرمجاز");
                swal({
                    type: 'warning',
                    title: 'درخواست غیرمجاز',
                    confirmButtonText: 'تایید!'
                });
            }
            else {
                alert('error.status ' + error.status);
            }
            // console.log('error---> ' + error);
            // console.log('error._body ' + error._body)
        });
        return ret.asObservable();
    }
}

let DataService = class DataService {
    constructor() {
        this.idValueSource = new BehaviorSubject(null);
        this.currentIdValue = this.idValueSource.asObservable();
    }
    changeIdValue(idValue) {
        this.idValueSource.next(idValue);
    }
};
DataService = __decorate([
    Injectable()
], DataService);

/**
 * @author Jafar Amini in March 2018.
 */
var RequestContentType;
(function (RequestContentType) {
    RequestContentType[RequestContentType["NONE"] = 0] = "NONE";
    RequestContentType[RequestContentType["JSON"] = 1] = "JSON";
    RequestContentType[RequestContentType["FORM"] = 2] = "FORM";
    RequestContentType[RequestContentType["FORM_DATA"] = 3] = "FORM_DATA";
    RequestContentType[RequestContentType["TEXT"] = 4] = "TEXT";
    RequestContentType[RequestContentType["BLOB"] = 5] = "BLOB";
    RequestContentType[RequestContentType["ARRAY_BUFFER"] = 6] = "ARRAY_BUFFER";
})(RequestContentType || (RequestContentType = {}));

/**
 * @author Jafar Amini in March 2018.
 */
var ResponseContentType;
(function (ResponseContentType) {
    ResponseContentType[ResponseContentType["Json"] = 0] = "Json";
    ResponseContentType[ResponseContentType["Text"] = 1] = "Text";
    ResponseContentType[ResponseContentType["Blob"] = 2] = "Blob";
    ResponseContentType[ResponseContentType["ArrayBuffer"] = 3] = "ArrayBuffer";
})(ResponseContentType || (ResponseContentType = {}));

class ServiceUtil {
    static getName(objectName, use) {
        if (isNullOrUndefined(use) || use !== false) {
            return (!isNullOrUndefined(objectName) && objectName !== '' ? '/' + objectName : '');
        }
        else {
            return '';
        }
    }
    static getRestOfUrlRespectTokenMode(objectName, restExtra, tokenMode, token) {
        if (isNullOrUndefined(token) && restExtra.needToken === true) {
            console.warn('token is not set.', '   angular-boot says: if called service does not need token you should set needToken: false in restExtra options');
        }
        if (tokenMode === TokenMode.HEADER || restExtra.needToken === false) {
            if (restExtra.urlQuery.length > 0) {
                return this.getName(restExtra.objectPrefix)
                    + this.getName(objectName)
                    + this.getName(restExtra.objectSuffix)
                    + '?'
                    + restExtra.urlQuery;
            }
            else {
                return this.getName(restExtra.objectPrefix)
                    + this.getName(objectName)
                    + this.getName(restExtra.objectSuffix);
            }
        }
        else if (tokenMode === TokenMode.URL) {
            return this.getName(restExtra.objectPrefix)
                + this.getName(objectName)
                + this.getName(restExtra.objectSuffix)
                + '?'
                + [restExtra.urlQuery, 'Authorization=' + token]
                    .filter(item => (item !== '')).join('&');
        }
    }
    static getRestOfUrl(objectName, restExtra) {
        return this.getName(restExtra.objectPrefix)
            + this.getName(objectName)
            + this.getName(restExtra.objectSuffix)
            + '?'
            + restExtra.urlQuery;
    }
    static getRestOfUrlWithAuthorization(objectName, restExtra, authorization) {
        return this.getName(restExtra.objectPrefix)
            + this.getName(objectName)
            + this.getName(restExtra.objectSuffix)
            + '?'
            + [restExtra.urlQuery, 'Authorization=' + authorization].join('&');
    }
    static setRequestContentType(headers, requestContentType) {
        switch (requestContentType) {
            case RequestContentType.NONE:
                return headers;
            case RequestContentType.JSON:
                return headers.append('Content-Type', 'application/json');
            case RequestContentType.FORM:
                return headers;
            case RequestContentType.FORM_DATA:
                // headers.delete('Content-Type');
                return headers;
            case RequestContentType.TEXT:
                return headers.append('Content-Type', 'text/xml');
            case RequestContentType.BLOB:
                return headers;
            case RequestContentType.ARRAY_BUFFER:
                return headers;
            default:
                return headers.append('Content-Type', 'application/json');
        }
    }
    static getResponseContentType(responseContentType) {
        switch (responseContentType) {
            case ResponseContentType.Json:
                return 'json';
            case ResponseContentType.Blob:
                return 'blob';
            case ResponseContentType.Text:
                return 'text';
            case ResponseContentType.ArrayBuffer:
                return 'arraybuffer';
            default:
                return 'json';
        }
    }
    // یک آبجکت ساده (فاقد تودرتویی) و یا یک آبجکت مرکب
    // (دارای تودرتویی) را می‌گیرد و رشته-کوئری مربوطه را برمی‌گرداند.
    static objectToQueryString2(object) {
        let finalResult = [];
        this.object2QueryString(finalResult, object);
        return finalResult.filter(item => (item !== '')).join('&');
    }
    static object2QueryString(finalResult, object) {
        if (isNullOrUndefined(object))
            return '';
        // let props: Pair1[];
        let props = new Array();
        Object.keys(object).forEach(key => {
            if (!isNullOrUndefined(object[key]) && object[key].toString().length > 0) {
                // alert('key:.... ' + key + '   object[key] = ' + object[key]);
                if (!isNullOrUndefined(object[key])) {
                    if (object[key].toString() === '[object Object]') {
                        // alert('Wawwwww');
                        this.object2QueryString(finalResult, object[key]);
                    }
                    else {
                        props.push(new PairKeyValue(key, object[key]));
                    }
                }
            }
            // console.log(key);
        });
        const result = [];
        for (let i = 0; i < props.length; i++) {
            // console.log(i + ' ---> ', props[i]);
            result.push([props[i].key, props[i].value]);
        }
        let r = result.map(param => param.join('=')).join('&');
        finalResult.push(r);
        return r;
    }
}

class ServiceBase {
    constructor() {
        this.prefixMatches = new Array();
        // this._url = this._ServiceConfig.getUrl();
        // this._url = this._ServiceConfig.getUrl(); // Config.getUrl();
        this._prefix = '';
    }
    addResponseAsBody(options) {
        options['observe'] = 'response';
    }
    getService(restExtra) {
        let headers1;
        let token = this._ServiceConfig.getToken(restExtra.keyToken);
        headers1 = this.resolveHeader(this._ServiceConfig, restExtra, token);
        let options = {
            headers: headers1,
            responseType: ServiceUtil.getResponseContentType(restExtra.responseContentType)
        };
        if (restExtra.responseAsBody === true) {
            this.addResponseAsBody(options);
        }
        return this._HttpClient
            .get(this._ServiceConfig.getUrl() +
            ServiceUtil.getRestOfUrlRespectTokenMode(this._objectName, restExtra, this._ServiceConfig.getTokenMode(), token), options)
            .pipe(map(res => res));
        // } else {
        //   return this._HttpClient
        //     .get(this._ServiceConfig.getUrl() +
        //       ServiceUtil.getRestOfUrlRespectTokenMode(
        //         this._objectName, restExtra,
        //         this._ServiceConfig.getTokenMode(), token),
        //       options);
        // }
    }
    postService(value, restExtra) {
        let headers1;
        let token = this._ServiceConfig.getToken(restExtra.keyToken);
        headers1 = this.resolveHeader(this._ServiceConfig, restExtra, token);
        let options = {
            headers: headers1,
            responseType: ServiceUtil.getResponseContentType(restExtra.responseContentType)
        };
        if (restExtra.responseAsBody === true) {
            this.addResponseAsBody(options);
        }
        return this._HttpClient.post(this._ServiceConfig.getUrl() + ServiceUtil
            .getRestOfUrlRespectTokenMode(this._objectName, restExtra, this._ServiceConfig.getTokenMode(), token) // + '?Authorization=' + this._ServiceConfig.getToken(restExtra.keyToken)
        , value, options).pipe(map(res => res));
        // } else {
        //   return this._HttpClient.post(this._ServiceConfig.getUrl() + ServiceUtil
        //     .getRestOfUrlRespectTokenMode(
        //       this._objectName, restExtra,
        //       this._ServiceConfig.getTokenMode(),
        //       token
        //     ) // + '?Authorization=' + this._ServiceConfig.getToken(restExtra.keyToken)
        //     , value,
        //     options);
        // }
    }
    putService(value, restExtra) {
        // Worked VVG
        // let headers = new HttpHeaders(
        //   {'Content-Type': 'application/json',
        //            'Authorization':
        //              this._ServiceConfig.getToken(restExtra.keyToken)
        //   }
        //   );
        let headers1;
        let token = this._ServiceConfig.getToken(restExtra.keyToken);
        headers1 = this.resolveHeader(this._ServiceConfig, restExtra, token);
        let options = {
            headers: headers1,
            responseType: ServiceUtil.getResponseContentType(restExtra.responseContentType)
        };
        if (restExtra.responseAsBody === true) {
            this.addResponseAsBody(options);
        }
        return this._HttpClient.put(this._ServiceConfig.getUrl() + ServiceUtil
            .getRestOfUrlRespectTokenMode(this._objectName, restExtra, this._ServiceConfig.getTokenMode(), token) // + '?Authorization=' + this._ServiceConfig.getToken(restExtra.keyToken)
        , value, options).pipe(map(res => res));
        // } else {
        //   return this._HttpClient.put(this._ServiceConfig.getUrl() + ServiceUtil
        //     .getRestOfUrlRespectTokenMode(
        //       this._objectName, restExtra,
        //       this._ServiceConfig.getTokenMode(),
        //       token
        //     ) // + '?Authorization=' + this._ServiceConfig.getToken(restExtra.keyToken)
        //     , value, options);
        // }
    }
    patchService(value, restExtra) {
        let headers1;
        let token = this._ServiceConfig.getToken(restExtra.keyToken);
        headers1 = this.resolveHeader(this._ServiceConfig, restExtra, token);
        let options = {
            headers: headers1,
            responseType: ServiceUtil.getResponseContentType(restExtra.responseContentType)
        };
        if (restExtra.responseAsBody === true) {
            this.addResponseAsBody(options);
        }
        return this._HttpClient.patch(this._ServiceConfig.getUrl() + ServiceUtil
            .getRestOfUrlRespectTokenMode(this._objectName, restExtra, this._ServiceConfig.getTokenMode(), token) // + '?Authorization=' + this._ServiceConfig.getToken(restExtra.keyToken)
        , value, options).pipe(map(res => res));
        // } else {
        //   return this._HttpClient.patch(this._ServiceConfig.getUrl() + ServiceUtil
        //     .getRestOfUrlRespectTokenMode(
        //       this._objectName, restExtra,
        //       this._ServiceConfig.getTokenMode(),
        //       token
        //     ) // + '?Authorization=' + this._ServiceConfig.getToken(restExtra.keyToken)
        //     , value, options);
        // }
    }
    deleteService(restExtra) {
        let headers1;
        let token = this._ServiceConfig.getToken(restExtra.keyToken);
        headers1 = this.resolveHeader(this._ServiceConfig, restExtra, token);
        let options = {
            headers: headers1,
            responseType: ServiceUtil.getResponseContentType(restExtra.responseContentType)
        };
        if (restExtra.responseAsBody === true) {
            this.addResponseAsBody(options);
        }
        return this._HttpClient.delete(this._ServiceConfig.getUrl() + ServiceUtil
            .getRestOfUrlRespectTokenMode(this._objectName, restExtra, this._ServiceConfig.getTokenMode(), token) // + '?Authorization=' + this._ServiceConfig.getToken(restExtra.keyToken)
        , options).pipe(map(res => res));
        // } else {
        //   return this._HttpClient.delete(this._ServiceConfig.getUrl() + ServiceUtil
        //     .getRestOfUrlRespectTokenMode(
        //       this._objectName, restExtra,
        //       this._ServiceConfig.getTokenMode(),
        //       token
        //     ) // + '?Authorization=' + this._ServiceConfig.getToken(restExtra.keyToken)
        //     , options);
        // }
    }
    resolveHeader(_ServiceConfig, restExtra, token) {
        let headers = new HttpHeaders();
        headers = ServiceUtil.setRequestContentType(headers, restExtra.requestContentType);
        if (_ServiceConfig.getTokenMode() === TokenMode.HEADER
            && restExtra.needToken === true) {
            if (isNullOrUndefined(token)) {
                console.warn('token is not set.', '   angular-boot says: if called service does not need token you should set needToken: false in restExtra options');
            }
            else {
                headers = headers.set('Authorization', token);
            }
        }
        if (!isNullOrUndefined(restExtra.includes) && restExtra.includes.length > 0) {
            headers = headers.append('includes', restExtra.includes);
        }
        else if (!isNullOrUndefined(restExtra.excludes) && restExtra.excludes.length > 0) {
            headers = headers.append('excludes', restExtra.excludes);
        }
        if (JSON.stringify(restExtra.otherHeaders) != '{}' &&
            !isNullOrUndefined(restExtra.otherHeaders)) {
            for (let key in restExtra.otherHeaders) {
                // console.log("      key:", key, "value:", restExtra.otherHeaders[key]);
                headers = headers.append(key, restExtra.otherHeaders[key]);
            }
        }
        return headers;
    }
}

// export class RestExtra {
//   use_prefix ? = true;
//   objectPrefix?: string;
//   objectSuffix?: string;
//   responseContentType?: ResponseContentType;
//   requestContentType?: RequestContentType;
//   urlQuery?: string;
//   // urlQuery: UrlQuery;
//   constructor(use_prefix, objectPrefix, objectSuffix, responseContentType,
//               requestContentType, urlQuery) {
//     this.use_prefix = use_prefix ? use_prefix : true;
//     this.objectPrefix = objectPrefix ? objectPrefix : '';
//     this.objectSuffix = objectSuffix ? objectSuffix : '';
//     this.requestContentType = requestContentType ? requestContentType : RequestContentType.JSON;
//     this.responseContentType = responseContentType ? responseContentType : ResponseContentType.Json;
//     this.urlQuery = urlQuery ? urlQuery : '';
//
//   }
// }
class RestExtra {
    // urlQuery: UrlQuery;
    constructor(options) {
        this.useObjectPrefix = true;
        this.useObjectSuffix = true;
        this.useObjectName = true;
        this.useObjectPrefix = options.useObjectPrefix ? options.useObjectPrefix : true;
        this.useObjectSuffix = options.useObjectSuffix ? options.useObjectSuffix : true;
        this.useObjectName = options.useObjectName ? options.useObjectName : true;
        this.objectPrefix = options.objectPrefix ? options.objectPrefix : '';
        this.objectSuffix = options.objectSuffix ? options.objectSuffix : '';
        this.requestContentType = options.requestContentType ? options.requestContentType : RequestContentType.JSON;
        this.responseContentType = options.responseContentType ? options.responseContentType : ResponseContentType.Json;
        this.urlQuery = options.urlQuery ? options.urlQuery : '';
        this.urlQueryObject = !isNullOrUndefined(options.urlQueryObject) ? options.urlQueryObject : {};
        this.includes = options.includes ? options.includes : null;
        this.excludes = options.excludes ? options.excludes : null;
        this.needToken = !isNullOrUndefined(options.needToken) ? options.needToken : true;
        this.keyToken = !isNullOrUndefined(options.keyToken) ? options.keyToken : null;
        this.otherHeaders = options.otherHeaders ? options.otherHeaders : {};
        this.restServiceResultPolicy = options.restServiceResultPolicy ?
            options.restServiceResultPolicy : null;
        this.restServiceErrorPolicy = options.restServiceErrorPolicy ?
            options.restServiceErrorPolicy : null;
        this.responseAsBody = !isNullOrUndefined(options.responseAsBody) ? options.responseAsBody : false;
    }
}
// export class RestExtraMini {
//   useObjectPrefix ? = true;
//   useObjectSuffix ? = true;
//   useObjectName ? = true;
//   responseContentType?: ResponseContentType;
//   requestContentType?: RequestContentType;
//   urlQuery?: string;
//   // urlQuery: UrlQuery;
//   constructor(use_prefix, responseContentType,
//               requestContentType, urlQuery) {
//     this.use_prefix = use_prefix ? use_prefix : true;
//     this.requestContentType = requestContentType ? requestContentType : RequestContentType.JSON;
//     this.responseContentType = responseContentType ? responseContentType : ResponseContentType.Json;
//     this.urlQuery = urlQuery ? urlQuery : '';
//
//   }
// }

class ResultHandle {
    static handleResponseContent(result) {
        // if (isNullOrUndefined(result) || isNullOrUndefined(result.flag)) {
        //   ret.next(result);
        // } else
        if (result.flag) {
            return result.data;
        }
        else if (!result.flag) {
            ErrorHandle.alertMessage(HandleResponse.getMessagesAsString(result.states, '<br>'));
        }
    }
}

/**
 * @author Jafar Amini in March 2018.
 */
// @Injectable()
class ServiceBase2 extends ServiceBase {
    constructor() {
        super();
        this.paramRegix = /\{(.*?)\}/g;
    }
    getMatches(_prefix) {
        return Toolkit2.StringUtil.getMatchesByRegix(_prefix, this.paramRegix);
    }
    setParams(str, matches, prefix) {
        for (let i = 0; i < matches[0].length; i++) {
            str = str.replace(matches[0][i], prefix[matches[1][i]]);
        }
        return str;
    }
    replaceParams(suffixPath, objectSuffix) {
        return this.setParams(suffixPath, this.getMatches(suffixPath), objectSuffix);
    }
    getPrefix(servicePrefixObject) {
        return this.setParams(this._prefix, this.prefixMatches, servicePrefixObject);
    }
    handleRestServiceResultPolicy(restExtra) {
        if (isNullOrUndefined(restExtra.restServiceResultPolicy)) {
            restExtra.restServiceResultPolicy =
                this._ServiceConfig.getRestServiceResultPolicy();
            if (isNullOrUndefined(restExtra.restServiceResultPolicy)) {
                restExtra.restServiceResultPolicy = RestServiceResultPolicy.NONE;
            }
        }
        return restExtra;
    }
    handleRestServiceErrorPolicy(restExtra) {
        if (isNullOrUndefined(restExtra.restServiceErrorPolicy)) {
            restExtra.restServiceErrorPolicy =
                this._ServiceConfig.getRestServiceErrorPolicy();
            if (isNullOrUndefined(restExtra.restServiceErrorPolicy)) {
                restExtra.restServiceErrorPolicy = RestServiceErrorPolicy.DEFAULT;
            }
        }
        return restExtra;
    }
    resolveRestExtra(restExtra) {
        if (isNullOrUndefined(restExtra)) {
            restExtra = this.handleRestServiceErrorPolicy(this.handleRestServiceResultPolicy(new RestExtra({})));
        }
        else {
            if (JSON.stringify(restExtra.urlQueryObject) != '{}' &&
                !isNullOrUndefined(restExtra.urlQueryObject)) {
                restExtra.urlQuery = ServiceUtil.objectToQueryString2(restExtra.urlQueryObject);
            }
            restExtra = new RestExtra(this.handleRestServiceErrorPolicy(this.handleRestServiceResultPolicy(restExtra)));
        }
        if (isNullOrUndefined(restExtra.objectPrefix) || restExtra.objectPrefix === '') {
            restExtra.objectPrefix = this.getPrefix({});
        }
        return restExtra;
    }
    handleResult(restExtra, res) {
        if (restExtra.restServiceResultPolicy === RestServiceResultPolicy.NONE) {
            return res;
        }
        else if (restExtra.restServiceResultPolicy === RestServiceResultPolicy.RESPONSE_CONTENT) {
            return ResultHandle.handleResponseContent(res);
            // if (res.flag) {
            //   ret.next(res.data);
            // } else if (!res.flag) {
            //   ErrorHandle.alertMessage(
            //     HandleResponse.getMessagesAsString(res.states, '<br>')
            //   );
            // }
        }
        else if (restExtra.restServiceResultPolicy === RestServiceResultPolicy.CUSTOM) {
            return this._ServiceConfig.applyCustomPolicyToResult(res);
        }
    }
    handleError(restExtra, error) {
        if (restExtra.restServiceErrorPolicy === RestServiceErrorPolicy.NONE) {
            return error;
        }
        else if (restExtra.restServiceErrorPolicy === RestServiceErrorPolicy.DEFAULT) {
            ErrorHandle.handleDefault(error, this._ServiceConfig);
            return error;
        }
        else if (restExtra.restServiceErrorPolicy === RestServiceErrorPolicy.CUSTOM) {
            return this._ServiceConfig.applyCustomPolicyToError(error);
        }
    }
    resetRestRequestStatus(options) {
        // if (options.resetStatusOnSuccess && options.resetStatusWaitTime > 0) {
        //   setTimeout(() => {
        //     options.status = 'not-started';
        //   }, options.resetStatusWaitTime);
        // }
    }
    getService(restExtra, options) {
        restExtra = this.resolveRestExtra(restExtra);
        let ret = new Subject();
        if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
            options.status = 'pending';
        }
        super.getService(restExtra).subscribe((res) => {
            if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
                options.status = 'success';
                // this.resetRestRequestStatus(options);
            }
            ret.next(this.handleResult(restExtra, res));
            // if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
            //   ret.next(res);
            // } else if (res.flag) {
            //   ret.next(res.data);
            // } else if (!res.flag) {
            //   ErrorHandle.alertMessage(
            //     HandleResponse.getMessagesAsString(res.states, '<br>')
            //   );
            // }
        }, error => {
            if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
                options.status = 'failure';
            }
            ret.error(this.handleError(restExtra, error));
        });
        return ret.asObservable();
    }
    // getService_f(restExtra?: RestExtra): Observable<any> {
    //   const ret = new Subject();
    //   super.getService(restExtra).subscribe((res) => {
    //     ret.next(res);
    //     // if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
    //     //   ret.next(res);
    //     // } else if (res.flag) {
    //     //   ret.next(res.data);
    //     // } else if (!res.flag) {
    //     //   ret.logicalError(res); //for future
    //     // }
    //   }, error => {
    //     ret.error(error);
    //   });
    //   return ret.asObservable();
    // }
    postService(value, restExtra, options) {
        restExtra = this.resolveRestExtra(restExtra);
        const ret = new Subject();
        if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
            options.status = 'pending';
        }
        super.postService(value, restExtra).subscribe((res) => {
            if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
                options.status = 'success';
                // this.resetRestRequestStatus(options);
            }
            ret.next(this.handleResult(restExtra, res));
            // console.log('response =======>', res);
            // if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
            //   ret.next(res);
            // } else if (res.flag) {
            //   ret.next(res.data);
            // } else if (!res.flag) {
            //   ErrorHandle.alertMessage(
            //     HandleResponse.getMessagesAsString(res.states, '<br>')
            //   );
            // }
        }, error => {
            if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
                options.status = 'failure';
            }
            ret.error(this.handleError(restExtra, error));
            // ErrorHandle.handleDefault(error, this._ServiceConfig);
            // ret.error(error);
        });
        return ret.asObservable();
    }
    // postService_f(value: any, restExtra?: RestExtra): Observable<any> {
    //   const ret = new Subject();
    //   super.postService(value, restExtra).subscribe((res) => {
    //     ret.next(res);
    //   }, error => {
    //     ret.error(error);
    //   });
    //   return ret.asObservable();
    // }
    putService(value, restExtra, options) {
        restExtra = this.resolveRestExtra(restExtra);
        const ret = new Subject();
        if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
            options.status = 'pending';
        }
        super.putService(value, restExtra).subscribe((res) => {
            if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
                options.status = 'success';
                // this.resetRestRequestStatus(options);
            }
            ret.next(this.handleResult(restExtra, res));
            // console.log('response =======>', res);
            // if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
            //   ret.next(res);
            // } else if (res.flag) {
            //   ret.next(res.data);
            // } else if (!res.flag) {
            //   ErrorHandle.alertMessage(
            //     HandleResponse.getMessagesAsString(res.states, '<br>')
            //   );
            // }
        }, error => {
            if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
                options.status = 'failure';
            }
            ret.error(this.handleError(restExtra, error));
            // ErrorHandle.handleDefault(error, this._ServiceConfig);
            // ret.error(error);
        });
        return ret.asObservable();
    }
    patchService(value, restExtra, options) {
        restExtra = this.resolveRestExtra(restExtra);
        const ret = new Subject();
        if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
            options.status = 'pending';
        }
        super.patchService(value, restExtra).subscribe((res) => {
            if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
                options.status = 'success';
                // this.resetRestRequestStatus(options);
            }
            ret.next(this.handleResult(restExtra, res));
        }, error => {
            if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
                options.status = 'failure';
            }
            ret.error(this.handleError(restExtra, error));
        });
        return ret.asObservable();
    }
    // putService_f(value: any, restExtra?: RestExtra): Observable<any> {
    //   const ret = new Subject();
    //   super.putService(value, restExtra).subscribe((res) => {
    //     ret.next(res);
    //   }, error => {
    //     ret.error(error);
    //   });
    //   return ret.asObservable();
    // }
    deleteService(restExtra, options) {
        restExtra = this.resolveRestExtra(restExtra);
        const ret = new Subject();
        if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
            options.status = 'pending';
        }
        super.deleteService(restExtra).subscribe((res) => {
            if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
                options.status = 'success';
                // this.resetRestRequestStatus(options);
            }
            ret.next(this.handleResult(restExtra, res));
            // if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
            //   ret.next(res);
            // } else if (res.flag) {
            //   ret.next(res.data);
            // } else if (!res.flag) {
            //   ErrorHandle.alertMessage(
            //     HandleResponse.getMessagesAsString(res.states, '<br>')
            //   );
            // }
        }, error => {
            if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
                options.status = 'failure';
            }
            ret.error(this.handleError(restExtra, error));
            // ErrorHandle.handleDefault(error, this._ServiceConfig);
            // ret.error(error);
        });
        return ret.asObservable();
    }
}

/**
 * @author Jafar Amini in March 2018.
 */
// import {ResponseContentType} from './response-content-type';
// import {RestExtra} from './rest-extra';
//
// // @Injectable()
class ExtendedServiceBase2 extends ServiceBase2 {
    constructor() {
        super();
    }
    g_getOne(objectSuffix, responseContentType) {
        return this.getService({ objectSuffix: objectSuffix, responseContentType: responseContentType });
    }
    // g_getOne_f(objectSuffix: string, responseContentType?: ResponseContentType): any {
    //   return this.getService_f(
    //     {objectSuffix: objectSuffix, responseContentType: responseContentType});
    // }
    g_createOne(item, objectSuffix, responseContentType) {
        return this.postService(item, { objectSuffix: objectSuffix, responseContentType: responseContentType });
    }
    // g_createOne_f(item: T, objectSuffix?: string, responseContentType?: ResponseContentType): any {
    //   return this.postService(item,
    //     {objectSuffix: objectSuffix, responseContentType: responseContentType});
    // }
    g_getList(objectSuffix, responseContentType) {
        return this.getService({ objectSuffix: objectSuffix, responseContentType: responseContentType });
    }
    // g_getList_f(objectSuffix?: string, responseContentType?: ResponseContentType): any {
    //   return this.getService_f(
    //     {objectSuffix: objectSuffix, responseContentType: responseContentType});
    // }
    g_deleteOne(objectSuffix, responseContentType) {
        return this.deleteService({ objectSuffix: objectSuffix, responseContentType: responseContentType });
    }
    // g_deleteOne_f(objectSuffix: string, responseContentType?: ResponseContentType): any {
    //   return this.deleteService_f(
    //     {objectSuffix: objectSuffix, responseContentType: responseContentType});
    // }
    g_updateOne(item, objectSuffix, responseContentType) {
        return this.putService(item, { objectSuffix: objectSuffix, responseContentType: responseContentType });
    }
    // g_updateOne_f(item: T, objectSuffix?: string, responseContentType?: ResponseContentType): any {
    //   return this.putService_f(item,
    //     {objectSuffix: objectSuffix, responseContentType: responseContentType});
    // }
    g_getListByParentId(objectSuffix, responseContentType) {
        return this.getService({ objectSuffix: 'parentId/' + objectSuffix, responseContentType: responseContentType });
    }
    //
    // g_getListByParentId_f(objectSuffix?: string, responseContentType?: ResponseContentType): any {
    //   return this.getService(
    //     {objectSuffix: 'parentId/' + objectSuffix, responseContentType: responseContentType});
    // }
    signIn(item, objectSuffix, responseContentType) {
        return this.postService(item, { objectSuffix: objectSuffix, responseContentType: responseContentType });
    }
}
//
//   getListByParentId(parentId: string, urlExtra?: UrlExtra, responseContentType?: ResponseContentType) {
//     if (urlExtra) {
//       if (responseContentType) {
//         return this.getService('parentId/' + parentId + '/' + urlExtra.path, responseContentType);
//       } else {
//         return this.getService('parentId/' + parentId + '/' + urlExtra.path);
//       }
//     } else {
//       if (responseContentType) {
//         return this.getService('parentId/' + parentId, responseContentType);
//       } else {
//         return this.getService('parentId/' + parentId);
//       }
//     }
//
//   }
//
//   getListByParentId_f(parentId: string, urlExtra?: UrlExtra, responseContentType?: ResponseContentType) {
//     if (urlExtra) {
//       if (responseContentType) {
//         return this.getService_f(parentId + '/' + urlExtra.path);
//       } else {
//         return this.getService_f(parentId + '/' + urlExtra.path);
//       }
//     } else {
//       if (responseContentType) {
//         return this.getService_f(parentId);
//       } else {
//         return this.getService_f(parentId);
//       }
//     }
//
//   }
//
//   signIn(item: T, urlExtra?: UrlExtra) {
//     return this.postService( urlExtra.path, item);
//   }
//
//
//   createOne_f(item: T, urlExtra?: UrlExtra) {
//     if (urlExtra) {
//       return this.postService_f(urlExtra.path, item);
//     } else {
//       return this.postService_f('', item);
//     }
//   }
//   getList_f(urlExtra?: UrlExtra, responseContentType?: ResponseContentType) {
//     if (urlExtra) {
//       if (responseContentType) {
//         return this.getService_f(urlExtra);
//       } else {
//         return this.getService_f(urlExtra);
//       }
//     } else {
//       if (responseContentType) {
//         return this.getService_f('');
//       } else {
//         return this.getService_f('');
//       }
//     }
//
//   }
//   deleteOne(itemId, urlExtra?: UrlExtra) {
//     if (urlExtra) {
//       return this.deleteService(itemId + '/' + urlExtra.path);
//     } else {
//       return this.deleteService(itemId);
//     }
//   }
//
//   deleteOne_f(itemId, urlExtra?: UrlExtra) {
//     if (urlExtra) {
//     return this.deleteService_f(itemId + '/' + urlExtra.path);
//     } else {
//       return this.deleteService_f(itemId);
//     }
//   }
// deleteOne_f(itemId, urlExtra?: UrlExtra) {
//   if (urlExtra) {
//     return this.deleteService_f(itemId + '/' + urlExtra.path);
//   } else {
//     return this.deleteService_f(itemId);
//   }
// }
//   getOne(itemId, urlExtra?: UrlExtra, responseContentType?: ResponseContentType): any {
//     if (urlExtra) {
//       if (responseContentType) {
//         return this.getService(itemId + '/' + urlExtra.path, responseContentType);
//       } else {
//         return this.getService(itemId + '/' + urlExtra.path);
//       }
//     } else {
//       if (responseContentType) {
//         return this.getService(itemId, responseContentType);
//       } else {
//         return this.getService(itemId);
//       }
//     }
//
//   }
//
//   getOne_f(itemId, urlExtra?: UrlExtra, responseContentType?: ResponseContentType): any {
//     if (urlExtra) {
//       if (responseContentType) {
//         return this.getService_f(itemId + '/' + urlExtra.path);
//       } else {
//         return this.getService_f(itemId + '/' + urlExtra.path);
//       }
//     } else {
//       if (responseContentType) {
//         return this.getService_f(itemId);
//       } else {
//         return this.getService_f(itemId);
//       }
//     }
//   }
//
//
//   updateOne(item: T, urlExtra?: UrlExtra, itemId?: string) {
//     if (urlExtra) {
//       if (itemId) {
//         return this.putService(itemId + '/' + urlExtra.path, item);
//       } else {
//         return this.putService( urlExtra.path, item);
//       }
//     } else {
//       if (itemId) {
//         return this.putService(itemId, item);
//       } else {
//         return this.putService('', item);
//       }
//     }
//
//   }
//
//   updateOne_f(item: T, urlExtra?: UrlExtra, itemId?: string) {
//     if (urlExtra) {
//       if (itemId) {
//         return this.putService_f(itemId + '/' + urlExtra.path, item);
//       } else {
//         return this.putService_f( urlExtra.path, item);
//       }
//     } else {
//       if (itemId) {
//         return this.putService_f(itemId, item);
//       } else {
//         return this.putService_f('', item);
//       }
//     }
//   }
//

/**
 * @author Jafar Amini in March 2018.
 */
// import {ResponseContentType} from './response-content-type';
// import {RestExtra} from './rest-extra';
//
// // @Injectable()
class HelperServiceBase2 extends ServiceBase2 {
    constructor() {
        super();
    }
    helperGetService(suffixPath, objectSuffix, prefix, restExtra) {
        return super.getService(this.getRestExtra(suffixPath, objectSuffix, prefix, restExtra));
    }
    helperDeleteService(suffixPath, objectSuffix, prefix, restExtra) {
        return this.deleteService(this.getRestExtra(suffixPath, objectSuffix, prefix, restExtra));
    }
    helperPutService(value, suffixPath, objectSuffix, prefix, restExtra) {
        return this.putService(value, this.getRestExtra(suffixPath, objectSuffix, prefix, restExtra));
    }
    helperPostService(value, suffixPath, objectSuffix, prefix, restExtra) {
        return this.postService(value, this.getRestExtra(suffixPath, objectSuffix, prefix, restExtra));
    }
    getRestExtra(suffixPath, objectSuffix, prefix, restExtra) {
        let myRestExtra;
        if (isNullOrUndefined(restExtra)) {
            myRestExtra = new RestExtra({});
        }
        else {
            myRestExtra = restExtra;
        }
        myRestExtra.objectSuffix = this.replaceParams(suffixPath, objectSuffix);
        myRestExtra.objectPrefix = this.getPrefix(prefix);
        return myRestExtra;
        // return {
        //   objectPrefix: this.getPrefix(prefix),
        //   objectSuffix: this.replaceParams(suffixPath, objectSuffix),
        //   requestContentType: isNullOrUndefined(restExtra) ? RequestContentType.JSON : restExtra.requestContentType,
        //   responseContentType: isNullOrUndefined(restExtra) ? ResponseContentType.Json : restExtra.responseContentType,
        //   urlQuery: isNullOrUndefined(restExtra) ? '' : restExtra.urlQuery,
        //   useObjectPrefix: isNullOrUndefined(restExtra) ? true : restExtra.useObjectPrefix,
        //   useObjectSuffix: isNullOrUndefined(restExtra) ? true : restExtra.useObjectSuffix,
        //   useObjectName: isNullOrUndefined(restExtra) ? true : restExtra.useObjectName
        // };
    }
}

let AssetsService = class AssetsService {
    constructor(_HttpClient) {
        this._HttpClient = _HttpClient;
    }
    getData(url, extra) {
        let myExtra = extra;
        if (isNullOrUndefined(extra)) {
            myExtra = { responseContentType: ResponseContentType.Json };
        }
        else if (isNullOrUndefined(extra.responseContentType)) {
            myExtra.responseContentType = ResponseContentType.Json;
        }
        return this._HttpClient.get(url, { responseType: ServiceUtil.getResponseContentType(myExtra.responseContentType) })
            .toPromise()
            .then((response) => {
            return response;
        }).catch((err) => {
            console.log(err);
            return null;
        });
    }
};
AssetsService.ctorParameters = () => [
    { type: HttpClient }
];
AssetsService.ngInjectableDef = ɵɵdefineInjectable({ factory: function AssetsService_Factory() { return new AssetsService(ɵɵinject(HttpClient)); }, token: AssetsService, providedIn: "root" });
AssetsService = __decorate([
    Injectable({
        providedIn: 'root'
    })
], AssetsService);

function isFunction(value) {
    return typeof value === 'function';
}
const takeUntilDestroyed = (componentInstance, destroyMethodName = 'ngOnDestroy') => (source) => {
    const originalDestroy = componentInstance[destroyMethodName];
    if (isFunction(originalDestroy) === false) {
        throw new Error(`${componentInstance.constructor.name} is using untilDestroyed but doesn't implement ${destroyMethodName}`);
    }
    if (!componentInstance['__takeUntilDestroy']) {
        componentInstance['__takeUntilDestroy'] = new Subject();
        componentInstance[destroyMethodName] = function () {
            // tslint:disable-next-line:no-unused-expression
            isFunction(originalDestroy) && originalDestroy.apply(this, arguments);
            componentInstance['__takeUntilDestroy'].next(true);
            componentInstance['__takeUntilDestroy'].complete();
        };
    }
    return source.pipe(takeUntil(componentInstance['__takeUntilDestroy']));
};

/*
 * Public API Surface of core
 */

/**
 * Generated bundle index. Do not edit.
 */

export { AssetsService, CacheModule, CacheService, Config, CoreComponent, CoreModule, CoreService, DataService, ErrorHandle, ExtendedServiceBase2, FakeData, FakeServiceBase, FakeServiceBase2, GlobalConfigurations, HandleResponse, HelperServiceBase2, Keyword, RequestContentType, ResponseContent, ResponseContentType, RestExtra, ServiceBase, ServiceBase2, ServiceConfig, ServiceUtil, State, StoredItem, TokenMode, takeUntilDestroyed, CacheLocalStorage as ɵa, CacheStorageAbstract as ɵb, CacheSessionStorage as ɵc, CacheMemoryStorage as ɵd };
//# sourceMappingURL=angular-boot-core.js.map
