import { __decorate, __extends, __values } from 'tslib';
import { ɵɵdefineInjectable, Injectable, Component, NgModule, ɵɵinject } from '@angular/core';
import { BehaviorSubject, Subject } from 'rxjs';
import { isNullOrUndefined, Toolkit2, Notify, NotifyConfig, PairKeyValue, RestServiceResultPolicy, RestServiceErrorPolicy } from '@angular-boot/util';
import { CommonModule } from '@angular/common';
import swal from 'sweetalert2';
import { HttpHeaders, HttpClient } from '@angular/common/http';
import { map, takeUntil } from 'rxjs/operators';

var CoreService = /** @class */ (function () {
    function CoreService() {
    }
    CoreService.ngInjectableDef = ɵɵdefineInjectable({ factory: function CoreService_Factory() { return new CoreService(); }, token: CoreService, providedIn: "root" });
    CoreService = __decorate([
        Injectable({
            providedIn: 'root'
        })
    ], CoreService);
    return CoreService;
}());

var CoreComponent = /** @class */ (function () {
    function CoreComponent() {
    }
    CoreComponent.prototype.ngOnInit = function () {
    };
    CoreComponent = __decorate([
        Component({
            selector: 'lib-core',
            template: "\n    <p>\n      core works!\n    </p>\n  "
        })
    ], CoreComponent);
    return CoreComponent;
}());

var CoreModule = /** @class */ (function () {
    function CoreModule() {
    }
    CoreModule = __decorate([
        NgModule({
            declarations: [CoreComponent],
            imports: [],
            exports: [CoreComponent]
        })
    ], CoreModule);
    return CoreModule;
}());

var StoredItem = /** @class */ (function () {
    function StoredItem(key) {
        this.value = new BehaviorSubject(null);
        this.exists = new BehaviorSubject(true);
        this.currentValue = this.value.asObservable();
        this.currentExists = this.exists.asObservable();
        this.key = key;
    }
    StoredItem.prototype.changeValue = function (value) {
        this.value.next(value);
    };
    StoredItem.prototype.changExists = function (value) {
        this.exists.next(value);
    };
    return StoredItem;
}());

/**
 * Abstract cache storage
 */
var CacheStorageAbstract = /** @class */ (function () {
    function CacheStorageAbstract() {
    }
    return CacheStorageAbstract;
}());

/**
 * Service for storing data in session storage
 */
var CacheSessionStorage = /** @class */ (function (_super) {
    __extends(CacheSessionStorage, _super);
    function CacheSessionStorage() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    CacheSessionStorage.prototype.getItem = function (key) {
        var value = sessionStorage.getItem(key);
        return value ? JSON.parse(value) : null;
    };
    CacheSessionStorage.prototype.setItem = function (key, value) {
        try {
            sessionStorage.setItem(key, JSON.stringify(value));
            return true;
        }
        catch (e) {
            return false;
        }
    };
    CacheSessionStorage.prototype.removeItem = function (key) {
        sessionStorage.removeItem(key);
    };
    CacheSessionStorage.prototype.clear = function () {
        sessionStorage.clear();
    };
    CacheSessionStorage.prototype.type = function () {
        return 1 /* SESSION_STORAGE */;
    };
    CacheSessionStorage.prototype.isEnabled = function () {
        try {
            var tmp = new Date().getTime();
            sessionStorage.setItem('test' + tmp, 'test');
            sessionStorage.removeItem('test' + tmp);
            return true;
        }
        catch (e) {
            return false;
        }
    };
    CacheSessionStorage.ngInjectableDef = ɵɵdefineInjectable({ factory: function CacheSessionStorage_Factory() { return new CacheSessionStorage(); }, token: CacheSessionStorage, providedIn: "root" });
    CacheSessionStorage = __decorate([
        Injectable({
            providedIn: 'root'
        })
    ], CacheSessionStorage);
    return CacheSessionStorage;
}(CacheStorageAbstract));

/**
 * Service for storing data in local storage
 */
var CacheLocalStorage = /** @class */ (function (_super) {
    __extends(CacheLocalStorage, _super);
    function CacheLocalStorage() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    CacheLocalStorage.prototype.getItem = function (key) {
        var value = localStorage.getItem(key);
        return value ? JSON.parse(value) : null;
    };
    CacheLocalStorage.prototype.setItem = function (key, value) {
        try {
            localStorage.setItem(key, JSON.stringify(value));
            return true;
        }
        catch (e) {
            return false;
        }
    };
    CacheLocalStorage.prototype.removeItem = function (key) {
        localStorage.removeItem(key);
    };
    CacheLocalStorage.prototype.clear = function () {
        localStorage.clear();
    };
    CacheLocalStorage.prototype.type = function () {
        return 0 /* LOCAL_STORAGE */;
    };
    CacheLocalStorage.prototype.isEnabled = function () {
        try {
            var tmp = new Date().getTime();
            localStorage.setItem('test' + tmp, 'test');
            localStorage.removeItem('test' + tmp);
            return true;
        }
        catch (e) {
            return false;
        }
    };
    CacheLocalStorage.ngInjectableDef = ɵɵdefineInjectable({ factory: function CacheLocalStorage_Factory() { return new CacheLocalStorage(); }, token: CacheLocalStorage, providedIn: "root" });
    CacheLocalStorage = __decorate([
        Injectable({
            providedIn: 'root'
        })
    ], CacheLocalStorage);
    return CacheLocalStorage;
}(CacheStorageAbstract));

/**
 * Service for storing data in local storage
 */
var CacheMemoryStorage = /** @class */ (function (_super) {
    __extends(CacheMemoryStorage, _super);
    function CacheMemoryStorage() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._data = {};
        return _this;
    }
    CacheMemoryStorage.prototype.getItem = function (key) {
        return this._data[key] ? this._data[key] : null;
    };
    CacheMemoryStorage.prototype.setItem = function (key, value) {
        this._data[key] = value;
        return true;
    };
    CacheMemoryStorage.prototype.removeItem = function (key) {
        delete this._data[key];
    };
    CacheMemoryStorage.prototype.clear = function () {
        this._data = [];
    };
    CacheMemoryStorage.prototype.type = function () {
        return 2 /* MEMORY */;
    };
    CacheMemoryStorage.prototype.isEnabled = function () {
        return true;
    };
    CacheMemoryStorage.ngInjectableDef = ɵɵdefineInjectable({ factory: function CacheMemoryStorage_Factory() { return new CacheMemoryStorage(); }, token: CacheMemoryStorage, providedIn: "root" });
    CacheMemoryStorage = __decorate([
        Injectable({
            providedIn: 'root'
        })
    ], CacheMemoryStorage);
    return CacheMemoryStorage;
}(CacheStorageAbstract));

// const CACHE_PREFIX = 'CacheService';
var CACHE_PREFIX = '';
var CacheService = /** @class */ (function () {
    function CacheService(localStorage, sessionStorage, memoryStorage) {
        this.localStorage = localStorage;
        this.sessionStorage = sessionStorage;
        this.memoryStorage = memoryStorage;
        /**
         * Cache prefix
         */
        this._prefix = CACHE_PREFIX;
        /**
         * Default cache options
         * CacheOptionsInterface
         */
        this._defaultOptions = {
            expires: Number.MAX_VALUE,
            maxAge: Number.MAX_VALUE
        };
        this.localStorageStoredItemList = [];
        this.sessionStorageStoredItemList = [];
        this.memoryStoredItemList = [];
    }
    // Start
    CacheService.prototype.setItem = function (key, value, storageType, options) {
        this.set(key, value, storageType, options);
        this.setToStoredList(key, value, storageType, true);
    };
    CacheService.prototype.getItem = function (key, storageType) {
        var val = this.get(key, storageType);
        if (val !== null) {
            if (isNullOrUndefined(this.getRelatedList(storageType).find(function (e) { return e.key === key; }))) {
                this.setToStoredList(key, val, storageType, true);
            }
            return this.getRelatedList(storageType).find(function (e) { return e.key === key; }).currentValue;
        }
        this.set(key, null, storageType);
        this.setToStoredList(key, null, storageType, false);
        return this.getRelatedList(storageType).find(function (e) { return e.key === key; }).currentValue;
    };
    CacheService.prototype.existsItem = function (key, storageType) {
        var val = this.get(key, storageType);
        if (val !== null) {
            if (isNullOrUndefined(this.getRelatedList(storageType).find(function (e) { return e.key === key; }))) {
                this.setToStoredList(key, val, storageType, true);
            }
            return this.getRelatedList(storageType).find(function (e) { return e.key === key; }).currentExists;
        }
        if (this.getRelatedList(storageType).find(function (e) { return e.key === key; }) === undefined) {
            return new BehaviorSubject(false);
        }
        return this.getRelatedList(storageType).find(function (e) { return e.key === key; }).currentExists;
    };
    CacheService.prototype.removeItem = function (key, storageType) {
        return this.remove(key, storageType);
    };
    // End
    CacheService.prototype.setToStoredList = function (key, value, storageType, exists) {
        this.getRelatedList(storageType).push(new StoredItem(key));
        this.getRelatedList(storageType).find(function (e) { return e.key === key; }).changeValue(value);
        this.getRelatedList(storageType).find(function (e) { return e.key === key; }).changExists(exists);
    };
    CacheService.prototype.getRelatedList = function (storageType) {
        switch (storageType) {
            case 0 /* LOCAL_STORAGE */:
                return this.localStorageStoredItemList;
                break;
            case 1 /* SESSION_STORAGE */:
                return this.sessionStorageStoredItemList;
                break;
            case 2 /* MEMORY */:
                return this.memoryStoredItemList;
                break;
        }
    };
    /**
     * Get data from cache
     * key
     * {any}
     */
    CacheService.prototype.get = function (key, storageType) {
        var storageValue = this.getMyValidatedStorage(storageType).getItem(this._toStorageKey(key));
        var value = null;
        if (storageValue) {
            if (this._validateStorageValue(storageValue)) {
                value = storageValue.value;
            }
            else {
                this.remove(key, storageType);
            }
        }
        return value;
    };
    CacheService.prototype.set = function (key, value, storageType, options) {
        var storageKey = this._toStorageKey(key);
        options = options ? options : this._defaultOptions;
        if (this.getMyValidatedStorage(storageType)
            .setItem(storageKey, this._toStorageValue(value, options))) {
            if (!this._isSystemKey(key) && options.tag) {
                this._saveTag(options.tag, storageKey, storageType);
            }
            return true;
        }
        return false;
    };
    /**
     * Check if value exists
     * key
     * {boolean}
     */
    CacheService.prototype.exists = function (key, storageType) {
        return !!this.get(key, storageType);
    };
    /**
     * Remove item from cache
     * key
     */
    CacheService.prototype.remove = function (key, storageType) {
        this.getMyValidatedStorage(storageType).removeItem(this._toStorageKey(key));
        this._removeFromTag(this._toStorageKey(key), storageType);
        this.removeFromStoredList(key, storageType);
    };
    /**
     * Remove key from tags keys list
     */
    CacheService.prototype._removeFromTag = function (key, storageType) {
        var tags = this.get(this._tagsStorageKey(), storageType) || {};
        var index;
        for (var tag in tags) {
            index = tags[tag].indexOf(key);
            if (index !== -1) {
                tags[tag].splice(index, 1);
                this.set(this._tagsStorageKey(), tags, storageType);
                break;
            }
        }
    };
    /**
     * Remove all from cache
     */
    CacheService.prototype.removeAll = function (storageType) {
        this.getMyValidatedStorage(storageType).clear();
    };
    /**
     * Get all tag data
     */
    CacheService.prototype.getTagData = function (tag, storageType) {
        var _this = this;
        var tags = this.get(this._tagsStorageKey(), storageType) || {};
        var result = {};
        if (tags[tag]) {
            tags[tag].forEach(function (key) {
                var data = _this.get(_this._fromStorageKey(key), storageType);
                if (data) {
                    result[_this._fromStorageKey(key)] = data;
                }
            });
        }
        return result;
    };
    /**
     * Remove all by tag
     */
    CacheService.prototype.removeTag = function (tag, storageType) {
        var _this = this;
        storageType = storageType ? storageType : this._defaultStorageType;
        var tags = this.get(this._tagsStorageKey(), storageType) || {};
        if (tags[tag]) {
            tags[tag].forEach(function (key) {
                _this.getMyValidatedStorage(storageType).removeItem(key);
            });
            delete tags[tag];
            this.set(this._tagsStorageKey(), tags, storageType);
        }
    };
    /**
     * Set global cache key prefix
     */
    CacheService.prototype.setGlobalPrefix = function (prefix) {
        this._prefix = prefix;
    };
    CacheService.prototype._toStorageKey = function (key) {
        return this._getCachePrefix() + key;
    };
    CacheService.prototype._fromStorageKey = function (key) {
        return key.replace(this._getCachePrefix(), '');
    };
    CacheService.prototype._getCachePrefix = function () {
        return this._prefix;
    };
    CacheService.prototype.getMyValidatedStorage = function (storageType) {
        switch (storageType) {
            case 2 /* MEMORY */:
                return this._validateStorage(this.memoryStorage);
                // return this.memoryStorage;
                break;
            case 0 /* LOCAL_STORAGE */:
                return this._validateStorage(this.localStorage);
                // return this.localStorage;
                break;
            case 1 /* SESSION_STORAGE */:
                return this._validateStorage(this.sessionStorage);
                // return this.sessionStorage;
                break;
        }
    };
    /**
     * Validate cache storage
     */
    CacheService.prototype._validateStorage = function (cacheStorage) {
        if (!cacheStorage.isEnabled()) {
            return this.memoryStorage;
        }
        return cacheStorage;
    };
    /**
     * Validate storage value
     */
    CacheService.prototype._validateStorageValue = function (value) {
        return !!value.options.expires && value.options.expires > Date.now();
    };
    /**
     * Prepare value to set to storage
     */
    CacheService.prototype._toStorageValue = function (value, options) {
        return {
            value: value,
            options: this._toStorageOptions(options)
        };
    };
    /**
     * Prepare options to set to storage
     */
    CacheService.prototype._toStorageOptions = function (options) {
        var storageOptions = {};
        storageOptions.expires = options.expires ? options.expires :
            (options.maxAge ? Date.now() + (options.maxAge * 1000) : this._defaultOptions.expires);
        storageOptions.maxAge = options.maxAge ? options.maxAge : this._defaultOptions.maxAge;
        return storageOptions;
    };
    /**
     * Save tag to list of tags
     */
    CacheService.prototype._saveTag = function (tag, key, storageType) {
        var tags = this.get(this._tagsStorageKey(), storageType) || {};
        if (!tags[tag]) {
            tags[tag] = [key];
        }
        else {
            tags[tag].push(key);
        }
        this.set(this._tagsStorageKey(), tags, storageType);
    };
    CacheService.prototype._tagsStorageKey = function () {
        // return 'CacheService_tags';
        return '';
    };
    /**
     * check if its system cache key
     */
    CacheService.prototype._isSystemKey = function (key) {
        return [this._tagsStorageKey()].indexOf(key) !== -1;
    };
    CacheService.prototype.removeFromStoredList = function (key, storageType) {
        this.getRelatedList(storageType).find(function (e) { return e.key === key; }).changeValue(null);
        this.getRelatedList(storageType).find(function (e) { return e.key === key; }).changExists(false);
        this.getRelatedList(storageType).filter(function (e) {
            return e.key !== key;
        });
    };
    CacheService.ctorParameters = function () { return [
        { type: CacheLocalStorage },
        { type: CacheSessionStorage },
        { type: CacheMemoryStorage }
    ]; };
    CacheService.ngInjectableDef = ɵɵdefineInjectable({ factory: function CacheService_Factory() { return new CacheService(ɵɵinject(CacheLocalStorage), ɵɵinject(CacheSessionStorage), ɵɵinject(CacheMemoryStorage)); }, token: CacheService, providedIn: "root" });
    CacheService = __decorate([
        Injectable({
            providedIn: 'root'
        })
    ], CacheService);
    return CacheService;
}());

var CacheModule = /** @class */ (function () {
    function CacheModule() {
    }
    CacheModule = __decorate([
        NgModule({
            declarations: [],
            imports: [
                CommonModule
            ],
            providers: [
                CacheService
            ]
        })
    ], CacheModule);
    return CacheModule;
}());

// export abstract class ServiceConfig {
//    abstract getUrl(): string;
//    abstract getToken(): string;
// }
var ServiceConfig = /** @class */ (function () {
    function ServiceConfig() {
    }
    return ServiceConfig;
}());

/**
 * @author Jafar Amini in March 2018.
 */
var TokenMode;
(function (TokenMode) {
    TokenMode[TokenMode["URL"] = 'URL'] = "URL";
    TokenMode[TokenMode["HEADER"] = 'HEADER'] = "HEADER";
})(TokenMode || (TokenMode = {}));

var GlobalConfigurations = /** @class */ (function () {
    function GlobalConfigurations() {
    }
    return GlobalConfigurations;
}());

/**
 * @author Jafar Amini in March 2018.
 */
var ErrorHandle = /** @class */ (function () {
    function ErrorHandle() {
    }
    ErrorHandle.handleDefault = function (error, _ServiceConfig) {
        console.error(error);
        switch (error.status) {
            case 0:
                Notify.showNotify(new NotifyConfig(Notify.Type.WARNING, Notify.Placement.TOP_LEFT, Notify.TEMPLATES.Template2, 'خطا در برقراری ارتباط با سرور ', 'خطا'));
                break;
            case 400:
                Notify.showNotify(new NotifyConfig(Notify.Type.WARNING, Notify.Placement.TOP_LEFT, Notify.TEMPLATES.Template2, 'اطلاعات ورودی درست نیست', 'خطا'));
                break;
            case 403:
                Notify.showNotify(new NotifyConfig(Notify.Type.WARNING, Notify.Placement.TOP_LEFT, Notify.TEMPLATES.Template2, 'درخواست غیرمجاز', 'توجه'));
                break;
            case 401:
                Notify.showNotify(new NotifyConfig(Notify.Type.WARNING, Notify.Placement.TOP_LEFT, Notify.TEMPLATES.Template2, 'نام کاربری یا رمز اشتباه است. یا اکانت فعال نشده است', 'توجه'));
                _ServiceConfig.redirectToLoginPage();
                break;
            case 404:
                Notify.showNotify(new NotifyConfig(Notify.Type.WARNING, Notify.Placement.TOP_LEFT, Notify.TEMPLATES.Template2, 'درخواست مورد نظر پیدا نشد', 'توجه'));
                break;
            default:
                var notifyConfig = new Toolkit2.NotifyConfig(Toolkit2.Notify.Type.DANGER, Toolkit2.Notify.Placement.TOP_LEFT, 
                // null,
                Toolkit2.Notify.TEMPLATES.Template2, 'خطایی رخ داد. کد خطا: '
                    + error.status, 'توجه شود');
                // notifyConfig.options.icon = 'fa fa-eye fa-2x';
                Toolkit2.Notify.showNotify(notifyConfig);
        }
    };
    ErrorHandle.alertMessage = function (messages) {
        swal({
            showCloseButton: true,
            confirmButtonColor: '#04383c',
            background: '#e4e4e4',
            type: 'error',
            title: 'خطا!',
            html: '<p>' + messages + '</p>',
            confirmButtonText: 'تایید'
        });
    };
    return ErrorHandle;
}());

/**
 * @author Jafar Amini in March 2018.
 */
var Config = /** @class */ (function () {
    function Config() {
    }
    Config.getFakeUrl = function () {
        return Config._fakeUrl;
    };
    Config.setFakeUrl = function (fakeUrl) {
        Config._fakeUrl = fakeUrl;
    };
    // public static getFakeUrl() {
    //   // return this._fakeUrl;
    //   return '/api/';
    // }
    Config._fakeUrl = '/api/';
    return Config;
}());

/**
 * @author Jafar Amini in March 2018.
 */
var FakeData = /** @class */ (function () {
    function FakeData() {
    }
    FakeData.getDomain = function (domainName) {
        var t = JSON.parse(localStorage.getItem(domainName));
        if (isNullOrUndefined(t)) {
            t = new Array();
        }
        return t;
    };
    return FakeData;
}());

/**
 * @author Jafar Amini in March 2018.
 */
// import {LocalStorage} from '../../../app/configuration/LocalStorage';
// @Injectable()
var FakeServiceBase = /** @class */ (function () {
    function FakeServiceBase() {
        this._prefix = '';
    }
    FakeServiceBase.prototype.getService = function (_objectsuffix) {
        var headers = new HttpHeaders();
        // headers =  headers.set('Authorization', LocalStorage.getEncodedBasicAuth());
        return this._HttpClient.get(this.url + this._prefix + this._objectName + '/'
            + _objectsuffix, { headers: headers });
    };
    FakeServiceBase.prototype.postService = function (value, _objectsuffix) {
        // console.log('finalUrl: ',this.url + this._prefix + this._objectName + '/' + _objectsuffix);
        var headers = new HttpHeaders();
        headers = headers
            .set('Content-Type', 'application/json');
        return this._HttpClient.post(this.url + this._prefix + this._objectName + '/' +
            _objectsuffix, value, { headers: headers });
    };
    FakeServiceBase.prototype.putService = function (value, _objectsuffix) {
        var headers = new HttpHeaders({ 'Content-Type': 'application/json'
            // , 'Authorization': LocalStorage.getEncodedBasicAuth()
        });
        return this._HttpClient.put(this.url + this._prefix + this._objectName + '/' +
            _objectsuffix
        // + '?Authorization=' + Config.getLocalStorageToken()
        , JSON.stringify(value), { headers: headers });
    };
    FakeServiceBase.prototype.deleteService = function (id) {
        var headers = new HttpHeaders();
        // headers = headers.set('Authorization', LocalStorage.getEncodedBasicAuth());
        return this._HttpClient.delete(this.url + this._prefix + this._objectName + '/' + (id), { headers: headers });
    };
    return FakeServiceBase;
}());

/**
 * @author Jafar Amini in March 2018.
 */
var Keyword;
(function (Keyword) {
    Keyword[Keyword["OK"] = 'OK'] = "OK";
    Keyword[Keyword["DELETED"] = 'با موفقیت حذف شد '] = "DELETED";
    Keyword[Keyword["NOT_FOUND"] = ' موردی یافت نشد'] = "NOT_FOUND";
    Keyword[Keyword["FAILURE"] = 'مشکلی رخ داد'] = "FAILURE";
    Keyword[Keyword["NOT_DONE"] = 'انجام نشد'] = "NOT_DONE";
    Keyword[Keyword["USER_NAME_DUPLICATE"] = 'نام کاربری تکراری است'] = "USER_NAME_DUPLICATE";
})(Keyword || (Keyword = {}));

var HandleResponse = /** @class */ (function () {
    function HandleResponse() {
    }
    HandleResponse.getMessagesAsString = function (states, separator) {
        var e_1, _a;
        var totalMessage = '';
        try {
            for (var states_1 = __values(states), states_1_1 = states_1.next(); !states_1_1.done; states_1_1 = states_1.next()) {
                var state = states_1_1.value;
                if (state.message !== '') {
                    totalMessage = totalMessage + separator + state.message;
                }
                else {
                    totalMessage = totalMessage + separator + Keyword[state.keyword];
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (states_1_1 && !states_1_1.done && (_a = states_1.return)) _a.call(states_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return totalMessage;
    };
    HandleResponse.getMessagesAsArray = function (states) {
        var e_2, _a;
        var totalMessage = new Array();
        try {
            for (var states_2 = __values(states), states_2_1 = states_2.next(); !states_2_1.done; states_2_1 = states_2.next()) {
                var state = states_2_1.value;
                if (state.message !== '') {
                    totalMessage.push(state.message);
                }
                else {
                    totalMessage.push(Keyword[state.keyword]);
                }
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (states_2_1 && !states_2_1.done && (_a = states_2.return)) _a.call(states_2);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return totalMessage;
    };
    return HandleResponse;
}());

var ResponseContent = /** @class */ (function () {
    function ResponseContent() {
    }
    return ResponseContent;
}());

var State = /** @class */ (function () {
    function State() {
    }
    return State;
}());

// @Injectable()
var FakeServiceBase2 = /** @class */ (function (_super) {
    __extends(FakeServiceBase2, _super);
    function FakeServiceBase2() {
        var _this = _super.call(this) || this;
        _this.url = Config.getFakeUrl();
        return _this;
    }
    FakeServiceBase2.prototype.getService = function (_url) {
        var ret = new Subject();
        _super.prototype.getService.call(this, _url).subscribe(function (res) {
            var e_1, _a;
            // console.log(_url + '------->' , res )
            // ;
            if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
                ret.next(res);
            }
            else if (res.flag) {
                ret.next(res.data);
            }
            else if (!res.flag) {
                // console.log('operation failed!');
                var from = 'top';
                var align = 'center';
                var type_ = 'danger';
                var msg = '';
                try {
                    // console.log('عملیات انجام نشد');
                    for (var _b = __values(res.states), _c = _b.next(); !_c.done; _c = _b.next()) {
                        var state = _c.value;
                        if (state.message !== '') {
                            // Notifyme.showNotify(from, align, type_, state.message);
                            // alert(state.message);
                            swal({
                                type: 'warning',
                                title: state.message,
                                confirmButtonText: 'تایید!'
                            });
                        }
                        else {
                            // Notifyme.showNotify(from, align, type_, Keyword[state.keyword]);
                            // alert(Keyword[state.keyword]);
                            swal({
                                type: 'warning',
                                title: Keyword[state.keyword],
                                confirmButtonText: 'تایید!'
                            });
                        }
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
            }
        }, function (error) {
            // if (error.status === 500){
            //   alert(500);
            //   alert(error._body)
            // }
            if (error.status === 403) {
                // alert("درخواست غیرمجاز");
                swal({
                    type: 'warning',
                    title: 'درخواست غیرمجاز',
                    confirmButtonText: 'تایید!'
                });
            }
            else {
                alert('error.status ' + error.status);
                // console.log('error---> ' + error);
                // console.log('error._body ' + error._body)
            }
        });
        return ret.asObservable();
    };
    FakeServiceBase2.prototype.getService_f = function (_url) {
        var ret = new Subject();
        _super.prototype.getService.call(this, _url).subscribe(function (res) {
            // console.log(_url + '------->' , res );
            if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
                ret.next(res);
            }
            else if (res.flag) {
                ret.next(res.data);
            }
            else if (!res.flag) {
                ret.error(res);
            }
        }, function (error) {
            // if (error.status === 500){
            //   alert(500);
            //   alert(error._body)
            // }
            if (error.status === 403) {
                swal({
                    type: 'warning',
                    title: 'درخواست غیرمجاز',
                    confirmButtonText: 'تایید!'
                });
            }
            else {
                alert('error.status ' + error.status);
                // console.log('error---> ' + error);
                // console.log('error._body ' + error._body)
            }
        });
        return ret.asObservable();
    };
    FakeServiceBase2.prototype.updateFakeDatabase = function (_objectName, objectData) {
        localStorage.setItem(_objectName, JSON.stringify(objectData));
    };
    FakeServiceBase2.prototype.postService = function (value, _url) {
        var _this = this;
        // console.log('value: ', value);
        var ret = new Subject();
        _super.prototype.postService.call(this, value, _url).subscribe(function (res) {
            var e_2, _a;
            _super.prototype.getService.call(_this, '').subscribe(function (res2) {
                // console.log('ressssssssss: ', res2);
                _this.updateFakeDatabase(_this._objectName, res2);
            });
            // console.log('response =======>', res);
            if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
                ret.next(res);
            }
            else if (res.flag) {
                ret.next(res.data);
            }
            else if (!res.flag) {
                // console.log('operation failed!');
                var from = 'top';
                var align = 'center';
                var type_ = 'danger';
                var msg = '';
                try {
                    for (var _b = __values(res.states), _c = _b.next(); !_c.done; _c = _b.next()) {
                        var state = _c.value;
                        if (state.message !== '') {
                            // Notifyme.showNotify(from, align, type_, state.message);
                            // alert(state.message);
                            swal({
                                showCloseButton: true,
                                confirmButtonColor: '#04383c',
                                background: '#e4e4e4',
                                type: 'error',
                                title: 'خطا!',
                                html: '<p>' + state.message + '</p>',
                                confirmButtonText: 'تایید'
                            });
                        }
                        else {
                            // Notifyme.showNotify(from, align, type_, Keyword[state.keyword]);
                            // alert(Keyword[state.keyword]);
                            swal({
                                showCloseButton: true,
                                confirmButtonColor: '#04383c',
                                background: '#e4e4e4',
                                type: 'error',
                                title: 'خطا!',
                                html: '<p>' + Keyword[state.keyword] + '</p>',
                                confirmButtonText: 'تایید'
                            });
                        }
                    }
                }
                catch (e_2_1) { e_2 = { error: e_2_1 }; }
                finally {
                    try {
                        if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                    }
                    finally { if (e_2) throw e_2.error; }
                }
            }
        }, function (error) {
            // if (error.status === 500){
            //   alert(500);
            //   alert(error._body)
            // }
            if (error.status === 403) {
                // alert("درخواست غیرمجاز");
                swal({
                    type: 'warning',
                    title: 'درخواست غیرمجاز',
                    confirmButtonText: 'تایید!'
                });
            }
            else {
                alert('error.status ' + error.status);
                // console.log('error---> ' + error);
                // console.log('error._body ' + error._body)
            }
        });
        return ret.asObservable();
    };
    FakeServiceBase2.prototype.postService_f = function (value, _url) {
        var ret = new Subject();
        _super.prototype.postService.call(this, value, _url).subscribe(function (res) {
            // console.log('response =======>', res);
            if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
                ret.next(res);
            }
            else if (res.flag) {
                ret.next(res.data);
            }
            else if (!res.flag) {
                ret.error(res);
            }
        }, function (error) {
            // if (error.status === 500){
            //   alert(500);
            //   alert(error._body)
            // }
            if (error.status === 403) {
                swal({
                    type: 'warning',
                    title: 'درخواست غیرمجاز',
                    confirmButtonText: 'تایید!'
                });
            }
            else {
                alert('error.status ' + error.status);
                // console.log('error---> ' + error);
                // console.log('error._body ' + error._body)
            }
        });
        return ret.asObservable();
    };
    FakeServiceBase2.prototype.putService = function (value, _url) {
        var ret = new Subject();
        _super.prototype.putService.call(this, value, _url).subscribe(function (res) {
            var e_3, _a;
            // console.log('response =======>', res);
            if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
                ret.next(res);
            }
            else if (res.flag) {
                ret.next(res.data);
            }
            else if (!res.flag) {
                // console.log('operation failed!');
                var from = 'top';
                var align = 'center';
                var type_ = 'danger';
                var msg = '';
                try {
                    for (var _b = __values(res.states), _c = _b.next(); !_c.done; _c = _b.next()) {
                        var state = _c.value;
                        if (state.message !== '') {
                            // Notifyme.showNotify(from, align, type_, state.message);
                            // alert(state.message);
                            swal({
                                type: 'warning',
                                title: state.message,
                                confirmButtonText: 'تایید!'
                            });
                        }
                        else {
                            // Notifyme.showNotify(from, align, type_, Keyword[state.keyword]);
                            // alert(Keyword[state.keyword]);
                            swal({
                                type: 'warning',
                                title: Keyword[state.keyword],
                                confirmButtonText: 'تایید!'
                            });
                        }
                    }
                }
                catch (e_3_1) { e_3 = { error: e_3_1 }; }
                finally {
                    try {
                        if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                    }
                    finally { if (e_3) throw e_3.error; }
                }
            }
        }, function (error) {
            // if (error.status === 500){
            //   alert(500);
            //   alert(error._body)
            // }
            if (error.status === 403) {
                // alert("درخواست غیرمجاز");
                swal({
                    type: 'warning',
                    title: 'درخواست غیرمجاز',
                    confirmButtonText: 'تایید!'
                });
            }
            else {
                alert('error.status ' + error.status);
                // console.log('error---> ' + error);
                // console.log('error._body ' + error._body)
            }
        });
        return ret.asObservable();
    };
    FakeServiceBase2.prototype.putService_f = function (value, _url) {
        var ret = new Subject();
        _super.prototype.putService.call(this, value, _url).subscribe(function (res) {
            // console.log('response =======>', res);
            if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
                ret.next(res);
            }
            else if (res.flag) {
                ret.next(res.data);
            }
            else if (!res.flag) {
                ret.error(res);
            }
        }, function (error) {
            // if (error.status === 500){
            //   alert(500);
            //   alert(error._body)
            // }
            if (error.status === 403) {
                // alert("درخواست غیرمجاز");
                swal({
                    type: 'warning',
                    title: 'درخواست غیرمجاز',
                    confirmButtonText: 'تایید!'
                });
            }
            else {
                alert('error.status ' + error.status);
            }
            // console.log('error---> ' + error);
            // console.log('error._body ' + error._body)
        });
        return ret.asObservable();
    };
    FakeServiceBase2.prototype.deleteService = function (parentId) {
        var _this = this;
        var ret = new Subject();
        _super.prototype.deleteService.call(this, parentId).subscribe(function (res) {
            // console.log('res---------------...>>', res);
            _super.prototype.getService.call(_this, '').subscribe(function (res2) {
                var e_4, _a;
                // console.log('ressssssssss: ', res2);
                _this.updateFakeDatabase(_this._objectName, res2);
                if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
                    ret.next(res);
                }
                else if (res.flag) {
                    ret.next(res.data);
                }
                else if (!res.flag) {
                    // console.log('operation failed!');
                    var from = 'top';
                    var align = 'center';
                    var type_ = 'danger';
                    var msg = '';
                    try {
                        for (var _b = __values(res.states), _c = _b.next(); !_c.done; _c = _b.next()) {
                            var state = _c.value;
                            if (state.message !== '') {
                                // Notifyme.showNotify(from, align, type_, state.message);
                                // alert(state.message);
                                swal({
                                    type: 'warning',
                                    title: state.message,
                                    confirmButtonText: 'تایید!'
                                });
                            }
                            else {
                                // Notifyme.showNotify(from, align, type_, Keyword[state.keyword]);
                                // alert(Keyword[state.keyword]);
                                swal({
                                    type: 'warning',
                                    title: Keyword[state.keyword],
                                    confirmButtonText: 'تایید!'
                                });
                            }
                        }
                    }
                    catch (e_4_1) { e_4 = { error: e_4_1 }; }
                    finally {
                        try {
                            if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                        }
                        finally { if (e_4) throw e_4.error; }
                    }
                }
            });
        }, function (error) {
            // if (error.status === 500){
            //   alert(500);
            //   alert(error._body)
            // }
            if (error.status === 403) {
                // alert("درخواست غیرمجاز");
                swal({
                    type: 'warning',
                    title: 'درخواست غیرمجاز',
                    confirmButtonText: 'تایید!'
                });
            }
            else {
                alert('error.status ' + error.status);
            }
            // console.log('error---> ' + error);
            // console.log('error._body ' + error._body)
        });
        return ret.asObservable();
    };
    FakeServiceBase2.prototype.deleteService_f = function (parentId) {
        var ret = new Subject();
        _super.prototype.deleteService.call(this, parentId).subscribe(function (res) {
            if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
                ret.next(res);
            }
            else if (res.flag) {
                ret.next(res.data);
            }
            else if (!res.flag) {
                ret.error(res);
            }
        }, function (error) {
            // if (error.status === 500){
            //   alert(500);
            //   alert(error._body)
            // }
            if (error.status === 403) {
                // alert("درخواست غیرمجاز");
                swal({
                    type: 'warning',
                    title: 'درخواست غیرمجاز',
                    confirmButtonText: 'تایید!'
                });
            }
            else {
                alert('error.status ' + error.status);
            }
            // console.log('error---> ' + error);
            // console.log('error._body ' + error._body)
        });
        return ret.asObservable();
    };
    return FakeServiceBase2;
}(FakeServiceBase));

var DataService = /** @class */ (function () {
    function DataService() {
        this.idValueSource = new BehaviorSubject(null);
        this.currentIdValue = this.idValueSource.asObservable();
    }
    DataService.prototype.changeIdValue = function (idValue) {
        this.idValueSource.next(idValue);
    };
    DataService = __decorate([
        Injectable()
    ], DataService);
    return DataService;
}());

/**
 * @author Jafar Amini in March 2018.
 */
var RequestContentType;
(function (RequestContentType) {
    RequestContentType[RequestContentType["NONE"] = 0] = "NONE";
    RequestContentType[RequestContentType["JSON"] = 1] = "JSON";
    RequestContentType[RequestContentType["FORM"] = 2] = "FORM";
    RequestContentType[RequestContentType["FORM_DATA"] = 3] = "FORM_DATA";
    RequestContentType[RequestContentType["TEXT"] = 4] = "TEXT";
    RequestContentType[RequestContentType["BLOB"] = 5] = "BLOB";
    RequestContentType[RequestContentType["ARRAY_BUFFER"] = 6] = "ARRAY_BUFFER";
})(RequestContentType || (RequestContentType = {}));

/**
 * @author Jafar Amini in March 2018.
 */
var ResponseContentType;
(function (ResponseContentType) {
    ResponseContentType[ResponseContentType["Json"] = 0] = "Json";
    ResponseContentType[ResponseContentType["Text"] = 1] = "Text";
    ResponseContentType[ResponseContentType["Blob"] = 2] = "Blob";
    ResponseContentType[ResponseContentType["ArrayBuffer"] = 3] = "ArrayBuffer";
})(ResponseContentType || (ResponseContentType = {}));

var ServiceUtil = /** @class */ (function () {
    function ServiceUtil() {
    }
    ServiceUtil.getName = function (objectName, use) {
        if (isNullOrUndefined(use) || use !== false) {
            return (!isNullOrUndefined(objectName) && objectName !== '' ? '/' + objectName : '');
        }
        else {
            return '';
        }
    };
    ServiceUtil.getRestOfUrlRespectTokenMode = function (objectName, restExtra, tokenMode, token) {
        if (isNullOrUndefined(token) && restExtra.needToken === true) {
            console.warn('token is not set.', '   angular-boot says: if called service does not need token you should set needToken: false in restExtra options');
        }
        if (tokenMode === TokenMode.HEADER || restExtra.needToken === false) {
            if (restExtra.urlQuery.length > 0) {
                return this.getName(restExtra.objectPrefix)
                    + this.getName(objectName)
                    + this.getName(restExtra.objectSuffix)
                    + '?'
                    + restExtra.urlQuery;
            }
            else {
                return this.getName(restExtra.objectPrefix)
                    + this.getName(objectName)
                    + this.getName(restExtra.objectSuffix);
            }
        }
        else if (tokenMode === TokenMode.URL) {
            return this.getName(restExtra.objectPrefix)
                + this.getName(objectName)
                + this.getName(restExtra.objectSuffix)
                + '?'
                + [restExtra.urlQuery, 'Authorization=' + token]
                    .filter(function (item) { return (item !== ''); }).join('&');
        }
    };
    ServiceUtil.getRestOfUrl = function (objectName, restExtra) {
        return this.getName(restExtra.objectPrefix)
            + this.getName(objectName)
            + this.getName(restExtra.objectSuffix)
            + '?'
            + restExtra.urlQuery;
    };
    ServiceUtil.getRestOfUrlWithAuthorization = function (objectName, restExtra, authorization) {
        return this.getName(restExtra.objectPrefix)
            + this.getName(objectName)
            + this.getName(restExtra.objectSuffix)
            + '?'
            + [restExtra.urlQuery, 'Authorization=' + authorization].join('&');
    };
    ServiceUtil.setRequestContentType = function (headers, requestContentType) {
        switch (requestContentType) {
            case RequestContentType.NONE:
                return headers;
            case RequestContentType.JSON:
                return headers.append('Content-Type', 'application/json');
            case RequestContentType.FORM:
                return headers;
            case RequestContentType.FORM_DATA:
                // headers.delete('Content-Type');
                return headers;
            case RequestContentType.TEXT:
                return headers.append('Content-Type', 'text/xml');
            case RequestContentType.BLOB:
                return headers;
            case RequestContentType.ARRAY_BUFFER:
                return headers;
            default:
                return headers.append('Content-Type', 'application/json');
        }
    };
    ServiceUtil.getResponseContentType = function (responseContentType) {
        switch (responseContentType) {
            case ResponseContentType.Json:
                return 'json';
            case ResponseContentType.Blob:
                return 'blob';
            case ResponseContentType.Text:
                return 'text';
            case ResponseContentType.ArrayBuffer:
                return 'arraybuffer';
            default:
                return 'json';
        }
    };
    // یک آبجکت ساده (فاقد تودرتویی) و یا یک آبجکت مرکب
    // (دارای تودرتویی) را می‌گیرد و رشته-کوئری مربوطه را برمی‌گرداند.
    ServiceUtil.objectToQueryString2 = function (object) {
        var finalResult = [];
        this.object2QueryString(finalResult, object);
        return finalResult.filter(function (item) { return (item !== ''); }).join('&');
    };
    ServiceUtil.object2QueryString = function (finalResult, object) {
        var _this = this;
        if (isNullOrUndefined(object))
            return '';
        // let props: Pair1[];
        var props = new Array();
        Object.keys(object).forEach(function (key) {
            if (!isNullOrUndefined(object[key]) && object[key].toString().length > 0) {
                // alert('key:.... ' + key + '   object[key] = ' + object[key]);
                if (!isNullOrUndefined(object[key])) {
                    if (object[key].toString() === '[object Object]') {
                        // alert('Wawwwww');
                        _this.object2QueryString(finalResult, object[key]);
                    }
                    else {
                        props.push(new PairKeyValue(key, object[key]));
                    }
                }
            }
            // console.log(key);
        });
        var result = [];
        for (var i = 0; i < props.length; i++) {
            // console.log(i + ' ---> ', props[i]);
            result.push([props[i].key, props[i].value]);
        }
        var r = result.map(function (param) { return param.join('='); }).join('&');
        finalResult.push(r);
        return r;
    };
    return ServiceUtil;
}());

var ServiceBase = /** @class */ (function () {
    function ServiceBase() {
        this.prefixMatches = new Array();
        // this._url = this._ServiceConfig.getUrl();
        // this._url = this._ServiceConfig.getUrl(); // Config.getUrl();
        this._prefix = '';
    }
    ServiceBase.prototype.addResponseAsBody = function (options) {
        options['observe'] = 'response';
    };
    ServiceBase.prototype.getService = function (restExtra) {
        var headers1;
        var token = this._ServiceConfig.getToken(restExtra.keyToken);
        headers1 = this.resolveHeader(this._ServiceConfig, restExtra, token);
        var options = {
            headers: headers1,
            responseType: ServiceUtil.getResponseContentType(restExtra.responseContentType)
        };
        if (restExtra.responseAsBody === true) {
            this.addResponseAsBody(options);
        }
        return this._HttpClient
            .get(this._ServiceConfig.getUrl() +
            ServiceUtil.getRestOfUrlRespectTokenMode(this._objectName, restExtra, this._ServiceConfig.getTokenMode(), token), options)
            .pipe(map(function (res) { return res; }));
        // } else {
        //   return this._HttpClient
        //     .get(this._ServiceConfig.getUrl() +
        //       ServiceUtil.getRestOfUrlRespectTokenMode(
        //         this._objectName, restExtra,
        //         this._ServiceConfig.getTokenMode(), token),
        //       options);
        // }
    };
    ServiceBase.prototype.postService = function (value, restExtra) {
        var headers1;
        var token = this._ServiceConfig.getToken(restExtra.keyToken);
        headers1 = this.resolveHeader(this._ServiceConfig, restExtra, token);
        var options = {
            headers: headers1,
            responseType: ServiceUtil.getResponseContentType(restExtra.responseContentType)
        };
        if (restExtra.responseAsBody === true) {
            this.addResponseAsBody(options);
        }
        return this._HttpClient.post(this._ServiceConfig.getUrl() + ServiceUtil
            .getRestOfUrlRespectTokenMode(this._objectName, restExtra, this._ServiceConfig.getTokenMode(), token) // + '?Authorization=' + this._ServiceConfig.getToken(restExtra.keyToken)
        , value, options).pipe(map(function (res) { return res; }));
        // } else {
        //   return this._HttpClient.post(this._ServiceConfig.getUrl() + ServiceUtil
        //     .getRestOfUrlRespectTokenMode(
        //       this._objectName, restExtra,
        //       this._ServiceConfig.getTokenMode(),
        //       token
        //     ) // + '?Authorization=' + this._ServiceConfig.getToken(restExtra.keyToken)
        //     , value,
        //     options);
        // }
    };
    ServiceBase.prototype.putService = function (value, restExtra) {
        // Worked VVG
        // let headers = new HttpHeaders(
        //   {'Content-Type': 'application/json',
        //            'Authorization':
        //              this._ServiceConfig.getToken(restExtra.keyToken)
        //   }
        //   );
        var headers1;
        var token = this._ServiceConfig.getToken(restExtra.keyToken);
        headers1 = this.resolveHeader(this._ServiceConfig, restExtra, token);
        var options = {
            headers: headers1,
            responseType: ServiceUtil.getResponseContentType(restExtra.responseContentType)
        };
        if (restExtra.responseAsBody === true) {
            this.addResponseAsBody(options);
        }
        return this._HttpClient.put(this._ServiceConfig.getUrl() + ServiceUtil
            .getRestOfUrlRespectTokenMode(this._objectName, restExtra, this._ServiceConfig.getTokenMode(), token) // + '?Authorization=' + this._ServiceConfig.getToken(restExtra.keyToken)
        , value, options).pipe(map(function (res) { return res; }));
        // } else {
        //   return this._HttpClient.put(this._ServiceConfig.getUrl() + ServiceUtil
        //     .getRestOfUrlRespectTokenMode(
        //       this._objectName, restExtra,
        //       this._ServiceConfig.getTokenMode(),
        //       token
        //     ) // + '?Authorization=' + this._ServiceConfig.getToken(restExtra.keyToken)
        //     , value, options);
        // }
    };
    ServiceBase.prototype.patchService = function (value, restExtra) {
        var headers1;
        var token = this._ServiceConfig.getToken(restExtra.keyToken);
        headers1 = this.resolveHeader(this._ServiceConfig, restExtra, token);
        var options = {
            headers: headers1,
            responseType: ServiceUtil.getResponseContentType(restExtra.responseContentType)
        };
        if (restExtra.responseAsBody === true) {
            this.addResponseAsBody(options);
        }
        return this._HttpClient.patch(this._ServiceConfig.getUrl() + ServiceUtil
            .getRestOfUrlRespectTokenMode(this._objectName, restExtra, this._ServiceConfig.getTokenMode(), token) // + '?Authorization=' + this._ServiceConfig.getToken(restExtra.keyToken)
        , value, options).pipe(map(function (res) { return res; }));
        // } else {
        //   return this._HttpClient.patch(this._ServiceConfig.getUrl() + ServiceUtil
        //     .getRestOfUrlRespectTokenMode(
        //       this._objectName, restExtra,
        //       this._ServiceConfig.getTokenMode(),
        //       token
        //     ) // + '?Authorization=' + this._ServiceConfig.getToken(restExtra.keyToken)
        //     , value, options);
        // }
    };
    ServiceBase.prototype.deleteService = function (restExtra) {
        var headers1;
        var token = this._ServiceConfig.getToken(restExtra.keyToken);
        headers1 = this.resolveHeader(this._ServiceConfig, restExtra, token);
        var options = {
            headers: headers1,
            responseType: ServiceUtil.getResponseContentType(restExtra.responseContentType)
        };
        if (restExtra.responseAsBody === true) {
            this.addResponseAsBody(options);
        }
        return this._HttpClient.delete(this._ServiceConfig.getUrl() + ServiceUtil
            .getRestOfUrlRespectTokenMode(this._objectName, restExtra, this._ServiceConfig.getTokenMode(), token) // + '?Authorization=' + this._ServiceConfig.getToken(restExtra.keyToken)
        , options).pipe(map(function (res) { return res; }));
        // } else {
        //   return this._HttpClient.delete(this._ServiceConfig.getUrl() + ServiceUtil
        //     .getRestOfUrlRespectTokenMode(
        //       this._objectName, restExtra,
        //       this._ServiceConfig.getTokenMode(),
        //       token
        //     ) // + '?Authorization=' + this._ServiceConfig.getToken(restExtra.keyToken)
        //     , options);
        // }
    };
    ServiceBase.prototype.resolveHeader = function (_ServiceConfig, restExtra, token) {
        var headers = new HttpHeaders();
        headers = ServiceUtil.setRequestContentType(headers, restExtra.requestContentType);
        if (_ServiceConfig.getTokenMode() === TokenMode.HEADER
            && restExtra.needToken === true) {
            if (isNullOrUndefined(token)) {
                console.warn('token is not set.', '   angular-boot says: if called service does not need token you should set needToken: false in restExtra options');
            }
            else {
                headers = headers.set('Authorization', token);
            }
        }
        if (!isNullOrUndefined(restExtra.includes) && restExtra.includes.length > 0) {
            headers = headers.append('includes', restExtra.includes);
        }
        else if (!isNullOrUndefined(restExtra.excludes) && restExtra.excludes.length > 0) {
            headers = headers.append('excludes', restExtra.excludes);
        }
        if (JSON.stringify(restExtra.otherHeaders) != '{}' &&
            !isNullOrUndefined(restExtra.otherHeaders)) {
            for (var key in restExtra.otherHeaders) {
                // console.log("      key:", key, "value:", restExtra.otherHeaders[key]);
                headers = headers.append(key, restExtra.otherHeaders[key]);
            }
        }
        return headers;
    };
    return ServiceBase;
}());

// export class RestExtra {
//   use_prefix ? = true;
//   objectPrefix?: string;
//   objectSuffix?: string;
//   responseContentType?: ResponseContentType;
//   requestContentType?: RequestContentType;
//   urlQuery?: string;
//   // urlQuery: UrlQuery;
//   constructor(use_prefix, objectPrefix, objectSuffix, responseContentType,
//               requestContentType, urlQuery) {
//     this.use_prefix = use_prefix ? use_prefix : true;
//     this.objectPrefix = objectPrefix ? objectPrefix : '';
//     this.objectSuffix = objectSuffix ? objectSuffix : '';
//     this.requestContentType = requestContentType ? requestContentType : RequestContentType.JSON;
//     this.responseContentType = responseContentType ? responseContentType : ResponseContentType.Json;
//     this.urlQuery = urlQuery ? urlQuery : '';
//
//   }
// }
var RestExtra = /** @class */ (function () {
    // urlQuery: UrlQuery;
    function RestExtra(options) {
        this.useObjectPrefix = true;
        this.useObjectSuffix = true;
        this.useObjectName = true;
        this.useObjectPrefix = options.useObjectPrefix ? options.useObjectPrefix : true;
        this.useObjectSuffix = options.useObjectSuffix ? options.useObjectSuffix : true;
        this.useObjectName = options.useObjectName ? options.useObjectName : true;
        this.objectPrefix = options.objectPrefix ? options.objectPrefix : '';
        this.objectSuffix = options.objectSuffix ? options.objectSuffix : '';
        this.requestContentType = options.requestContentType ? options.requestContentType : RequestContentType.JSON;
        this.responseContentType = options.responseContentType ? options.responseContentType : ResponseContentType.Json;
        this.urlQuery = options.urlQuery ? options.urlQuery : '';
        this.urlQueryObject = !isNullOrUndefined(options.urlQueryObject) ? options.urlQueryObject : {};
        this.includes = options.includes ? options.includes : null;
        this.excludes = options.excludes ? options.excludes : null;
        this.needToken = !isNullOrUndefined(options.needToken) ? options.needToken : true;
        this.keyToken = !isNullOrUndefined(options.keyToken) ? options.keyToken : null;
        this.otherHeaders = options.otherHeaders ? options.otherHeaders : {};
        this.restServiceResultPolicy = options.restServiceResultPolicy ?
            options.restServiceResultPolicy : null;
        this.restServiceErrorPolicy = options.restServiceErrorPolicy ?
            options.restServiceErrorPolicy : null;
        this.responseAsBody = !isNullOrUndefined(options.responseAsBody) ? options.responseAsBody : false;
    }
    return RestExtra;
}());
// export class RestExtraMini {
//   useObjectPrefix ? = true;
//   useObjectSuffix ? = true;
//   useObjectName ? = true;
//   responseContentType?: ResponseContentType;
//   requestContentType?: RequestContentType;
//   urlQuery?: string;
//   // urlQuery: UrlQuery;
//   constructor(use_prefix, responseContentType,
//               requestContentType, urlQuery) {
//     this.use_prefix = use_prefix ? use_prefix : true;
//     this.requestContentType = requestContentType ? requestContentType : RequestContentType.JSON;
//     this.responseContentType = responseContentType ? responseContentType : ResponseContentType.Json;
//     this.urlQuery = urlQuery ? urlQuery : '';
//
//   }
// }

var ResultHandle = /** @class */ (function () {
    function ResultHandle() {
    }
    ResultHandle.handleResponseContent = function (result) {
        // if (isNullOrUndefined(result) || isNullOrUndefined(result.flag)) {
        //   ret.next(result);
        // } else
        if (result.flag) {
            return result.data;
        }
        else if (!result.flag) {
            ErrorHandle.alertMessage(HandleResponse.getMessagesAsString(result.states, '<br>'));
        }
    };
    return ResultHandle;
}());

// @Injectable()
var ServiceBase2 = /** @class */ (function (_super) {
    __extends(ServiceBase2, _super);
    function ServiceBase2() {
        var _this = _super.call(this) || this;
        _this.paramRegix = /\{(.*?)\}/g;
        return _this;
    }
    ServiceBase2.prototype.getMatches = function (_prefix) {
        return Toolkit2.StringUtil.getMatchesByRegix(_prefix, this.paramRegix);
    };
    ServiceBase2.prototype.setParams = function (str, matches, prefix) {
        for (var i = 0; i < matches[0].length; i++) {
            str = str.replace(matches[0][i], prefix[matches[1][i]]);
        }
        return str;
    };
    ServiceBase2.prototype.replaceParams = function (suffixPath, objectSuffix) {
        return this.setParams(suffixPath, this.getMatches(suffixPath), objectSuffix);
    };
    ServiceBase2.prototype.getPrefix = function (servicePrefixObject) {
        return this.setParams(this._prefix, this.prefixMatches, servicePrefixObject);
    };
    ServiceBase2.prototype.handleRestServiceResultPolicy = function (restExtra) {
        if (isNullOrUndefined(restExtra.restServiceResultPolicy)) {
            restExtra.restServiceResultPolicy =
                this._ServiceConfig.getRestServiceResultPolicy();
            if (isNullOrUndefined(restExtra.restServiceResultPolicy)) {
                restExtra.restServiceResultPolicy = RestServiceResultPolicy.NONE;
            }
        }
        return restExtra;
    };
    ServiceBase2.prototype.handleRestServiceErrorPolicy = function (restExtra) {
        if (isNullOrUndefined(restExtra.restServiceErrorPolicy)) {
            restExtra.restServiceErrorPolicy =
                this._ServiceConfig.getRestServiceErrorPolicy();
            if (isNullOrUndefined(restExtra.restServiceErrorPolicy)) {
                restExtra.restServiceErrorPolicy = RestServiceErrorPolicy.DEFAULT;
            }
        }
        return restExtra;
    };
    ServiceBase2.prototype.resolveRestExtra = function (restExtra) {
        if (isNullOrUndefined(restExtra)) {
            restExtra = this.handleRestServiceErrorPolicy(this.handleRestServiceResultPolicy(new RestExtra({})));
        }
        else {
            if (JSON.stringify(restExtra.urlQueryObject) != '{}' &&
                !isNullOrUndefined(restExtra.urlQueryObject)) {
                restExtra.urlQuery = ServiceUtil.objectToQueryString2(restExtra.urlQueryObject);
            }
            restExtra = new RestExtra(this.handleRestServiceErrorPolicy(this.handleRestServiceResultPolicy(restExtra)));
        }
        if (isNullOrUndefined(restExtra.objectPrefix) || restExtra.objectPrefix === '') {
            restExtra.objectPrefix = this.getPrefix({});
        }
        return restExtra;
    };
    ServiceBase2.prototype.handleResult = function (restExtra, res) {
        if (restExtra.restServiceResultPolicy === RestServiceResultPolicy.NONE) {
            return res;
        }
        else if (restExtra.restServiceResultPolicy === RestServiceResultPolicy.RESPONSE_CONTENT) {
            return ResultHandle.handleResponseContent(res);
            // if (res.flag) {
            //   ret.next(res.data);
            // } else if (!res.flag) {
            //   ErrorHandle.alertMessage(
            //     HandleResponse.getMessagesAsString(res.states, '<br>')
            //   );
            // }
        }
        else if (restExtra.restServiceResultPolicy === RestServiceResultPolicy.CUSTOM) {
            return this._ServiceConfig.applyCustomPolicyToResult(res);
        }
    };
    ServiceBase2.prototype.handleError = function (restExtra, error) {
        if (restExtra.restServiceErrorPolicy === RestServiceErrorPolicy.NONE) {
            return error;
        }
        else if (restExtra.restServiceErrorPolicy === RestServiceErrorPolicy.DEFAULT) {
            ErrorHandle.handleDefault(error, this._ServiceConfig);
            return error;
        }
        else if (restExtra.restServiceErrorPolicy === RestServiceErrorPolicy.CUSTOM) {
            return this._ServiceConfig.applyCustomPolicyToError(error);
        }
    };
    ServiceBase2.prototype.resetRestRequestStatus = function (options) {
        // if (options.resetStatusOnSuccess && options.resetStatusWaitTime > 0) {
        //   setTimeout(() => {
        //     options.status = 'not-started';
        //   }, options.resetStatusWaitTime);
        // }
    };
    ServiceBase2.prototype.getService = function (restExtra, options) {
        var _this = this;
        restExtra = this.resolveRestExtra(restExtra);
        var ret = new Subject();
        if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
            options.status = 'pending';
        }
        _super.prototype.getService.call(this, restExtra).subscribe(function (res) {
            if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
                options.status = 'success';
                // this.resetRestRequestStatus(options);
            }
            ret.next(_this.handleResult(restExtra, res));
            // if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
            //   ret.next(res);
            // } else if (res.flag) {
            //   ret.next(res.data);
            // } else if (!res.flag) {
            //   ErrorHandle.alertMessage(
            //     HandleResponse.getMessagesAsString(res.states, '<br>')
            //   );
            // }
        }, function (error) {
            if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
                options.status = 'failure';
            }
            ret.error(_this.handleError(restExtra, error));
        });
        return ret.asObservable();
    };
    // getService_f(restExtra?: RestExtra): Observable<any> {
    //   const ret = new Subject();
    //   super.getService(restExtra).subscribe((res) => {
    //     ret.next(res);
    //     // if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
    //     //   ret.next(res);
    //     // } else if (res.flag) {
    //     //   ret.next(res.data);
    //     // } else if (!res.flag) {
    //     //   ret.logicalError(res); //for future
    //     // }
    //   }, error => {
    //     ret.error(error);
    //   });
    //   return ret.asObservable();
    // }
    ServiceBase2.prototype.postService = function (value, restExtra, options) {
        var _this = this;
        restExtra = this.resolveRestExtra(restExtra);
        var ret = new Subject();
        if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
            options.status = 'pending';
        }
        _super.prototype.postService.call(this, value, restExtra).subscribe(function (res) {
            if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
                options.status = 'success';
                // this.resetRestRequestStatus(options);
            }
            ret.next(_this.handleResult(restExtra, res));
            // console.log('response =======>', res);
            // if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
            //   ret.next(res);
            // } else if (res.flag) {
            //   ret.next(res.data);
            // } else if (!res.flag) {
            //   ErrorHandle.alertMessage(
            //     HandleResponse.getMessagesAsString(res.states, '<br>')
            //   );
            // }
        }, function (error) {
            if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
                options.status = 'failure';
            }
            ret.error(_this.handleError(restExtra, error));
            // ErrorHandle.handleDefault(error, this._ServiceConfig);
            // ret.error(error);
        });
        return ret.asObservable();
    };
    // postService_f(value: any, restExtra?: RestExtra): Observable<any> {
    //   const ret = new Subject();
    //   super.postService(value, restExtra).subscribe((res) => {
    //     ret.next(res);
    //   }, error => {
    //     ret.error(error);
    //   });
    //   return ret.asObservable();
    // }
    ServiceBase2.prototype.putService = function (value, restExtra, options) {
        var _this = this;
        restExtra = this.resolveRestExtra(restExtra);
        var ret = new Subject();
        if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
            options.status = 'pending';
        }
        _super.prototype.putService.call(this, value, restExtra).subscribe(function (res) {
            if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
                options.status = 'success';
                // this.resetRestRequestStatus(options);
            }
            ret.next(_this.handleResult(restExtra, res));
            // console.log('response =======>', res);
            // if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
            //   ret.next(res);
            // } else if (res.flag) {
            //   ret.next(res.data);
            // } else if (!res.flag) {
            //   ErrorHandle.alertMessage(
            //     HandleResponse.getMessagesAsString(res.states, '<br>')
            //   );
            // }
        }, function (error) {
            if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
                options.status = 'failure';
            }
            ret.error(_this.handleError(restExtra, error));
            // ErrorHandle.handleDefault(error, this._ServiceConfig);
            // ret.error(error);
        });
        return ret.asObservable();
    };
    ServiceBase2.prototype.patchService = function (value, restExtra, options) {
        var _this = this;
        restExtra = this.resolveRestExtra(restExtra);
        var ret = new Subject();
        if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
            options.status = 'pending';
        }
        _super.prototype.patchService.call(this, value, restExtra).subscribe(function (res) {
            if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
                options.status = 'success';
                // this.resetRestRequestStatus(options);
            }
            ret.next(_this.handleResult(restExtra, res));
        }, function (error) {
            if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
                options.status = 'failure';
            }
            ret.error(_this.handleError(restExtra, error));
        });
        return ret.asObservable();
    };
    // putService_f(value: any, restExtra?: RestExtra): Observable<any> {
    //   const ret = new Subject();
    //   super.putService(value, restExtra).subscribe((res) => {
    //     ret.next(res);
    //   }, error => {
    //     ret.error(error);
    //   });
    //   return ret.asObservable();
    // }
    ServiceBase2.prototype.deleteService = function (restExtra, options) {
        var _this = this;
        restExtra = this.resolveRestExtra(restExtra);
        var ret = new Subject();
        if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
            options.status = 'pending';
        }
        _super.prototype.deleteService.call(this, restExtra).subscribe(function (res) {
            if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
                options.status = 'success';
                // this.resetRestRequestStatus(options);
            }
            ret.next(_this.handleResult(restExtra, res));
            // if (isNullOrUndefined(res) || isNullOrUndefined(res.flag)) {
            //   ret.next(res);
            // } else if (res.flag) {
            //   ret.next(res.data);
            // } else if (!res.flag) {
            //   ErrorHandle.alertMessage(
            //     HandleResponse.getMessagesAsString(res.states, '<br>')
            //   );
            // }
        }, function (error) {
            if (!isNullOrUndefined(options) && !isNullOrUndefined(options.status)) {
                options.status = 'failure';
            }
            ret.error(_this.handleError(restExtra, error));
            // ErrorHandle.handleDefault(error, this._ServiceConfig);
            // ret.error(error);
        });
        return ret.asObservable();
    };
    return ServiceBase2;
}(ServiceBase));

// import {ResponseContentType} from './response-content-type';
// import {RestExtra} from './rest-extra';
//
// // @Injectable()
var ExtendedServiceBase2 = /** @class */ (function (_super) {
    __extends(ExtendedServiceBase2, _super);
    function ExtendedServiceBase2() {
        return _super.call(this) || this;
    }
    ExtendedServiceBase2.prototype.g_getOne = function (objectSuffix, responseContentType) {
        return this.getService({ objectSuffix: objectSuffix, responseContentType: responseContentType });
    };
    // g_getOne_f(objectSuffix: string, responseContentType?: ResponseContentType): any {
    //   return this.getService_f(
    //     {objectSuffix: objectSuffix, responseContentType: responseContentType});
    // }
    ExtendedServiceBase2.prototype.g_createOne = function (item, objectSuffix, responseContentType) {
        return this.postService(item, { objectSuffix: objectSuffix, responseContentType: responseContentType });
    };
    // g_createOne_f(item: T, objectSuffix?: string, responseContentType?: ResponseContentType): any {
    //   return this.postService(item,
    //     {objectSuffix: objectSuffix, responseContentType: responseContentType});
    // }
    ExtendedServiceBase2.prototype.g_getList = function (objectSuffix, responseContentType) {
        return this.getService({ objectSuffix: objectSuffix, responseContentType: responseContentType });
    };
    // g_getList_f(objectSuffix?: string, responseContentType?: ResponseContentType): any {
    //   return this.getService_f(
    //     {objectSuffix: objectSuffix, responseContentType: responseContentType});
    // }
    ExtendedServiceBase2.prototype.g_deleteOne = function (objectSuffix, responseContentType) {
        return this.deleteService({ objectSuffix: objectSuffix, responseContentType: responseContentType });
    };
    // g_deleteOne_f(objectSuffix: string, responseContentType?: ResponseContentType): any {
    //   return this.deleteService_f(
    //     {objectSuffix: objectSuffix, responseContentType: responseContentType});
    // }
    ExtendedServiceBase2.prototype.g_updateOne = function (item, objectSuffix, responseContentType) {
        return this.putService(item, { objectSuffix: objectSuffix, responseContentType: responseContentType });
    };
    // g_updateOne_f(item: T, objectSuffix?: string, responseContentType?: ResponseContentType): any {
    //   return this.putService_f(item,
    //     {objectSuffix: objectSuffix, responseContentType: responseContentType});
    // }
    ExtendedServiceBase2.prototype.g_getListByParentId = function (objectSuffix, responseContentType) {
        return this.getService({ objectSuffix: 'parentId/' + objectSuffix, responseContentType: responseContentType });
    };
    //
    // g_getListByParentId_f(objectSuffix?: string, responseContentType?: ResponseContentType): any {
    //   return this.getService(
    //     {objectSuffix: 'parentId/' + objectSuffix, responseContentType: responseContentType});
    // }
    ExtendedServiceBase2.prototype.signIn = function (item, objectSuffix, responseContentType) {
        return this.postService(item, { objectSuffix: objectSuffix, responseContentType: responseContentType });
    };
    return ExtendedServiceBase2;
}(ServiceBase2));
//
//   getListByParentId(parentId: string, urlExtra?: UrlExtra, responseContentType?: ResponseContentType) {
//     if (urlExtra) {
//       if (responseContentType) {
//         return this.getService('parentId/' + parentId + '/' + urlExtra.path, responseContentType);
//       } else {
//         return this.getService('parentId/' + parentId + '/' + urlExtra.path);
//       }
//     } else {
//       if (responseContentType) {
//         return this.getService('parentId/' + parentId, responseContentType);
//       } else {
//         return this.getService('parentId/' + parentId);
//       }
//     }
//
//   }
//
//   getListByParentId_f(parentId: string, urlExtra?: UrlExtra, responseContentType?: ResponseContentType) {
//     if (urlExtra) {
//       if (responseContentType) {
//         return this.getService_f(parentId + '/' + urlExtra.path);
//       } else {
//         return this.getService_f(parentId + '/' + urlExtra.path);
//       }
//     } else {
//       if (responseContentType) {
//         return this.getService_f(parentId);
//       } else {
//         return this.getService_f(parentId);
//       }
//     }
//
//   }
//
//   signIn(item: T, urlExtra?: UrlExtra) {
//     return this.postService( urlExtra.path, item);
//   }
//
//
//   createOne_f(item: T, urlExtra?: UrlExtra) {
//     if (urlExtra) {
//       return this.postService_f(urlExtra.path, item);
//     } else {
//       return this.postService_f('', item);
//     }
//   }
//   getList_f(urlExtra?: UrlExtra, responseContentType?: ResponseContentType) {
//     if (urlExtra) {
//       if (responseContentType) {
//         return this.getService_f(urlExtra);
//       } else {
//         return this.getService_f(urlExtra);
//       }
//     } else {
//       if (responseContentType) {
//         return this.getService_f('');
//       } else {
//         return this.getService_f('');
//       }
//     }
//
//   }
//   deleteOne(itemId, urlExtra?: UrlExtra) {
//     if (urlExtra) {
//       return this.deleteService(itemId + '/' + urlExtra.path);
//     } else {
//       return this.deleteService(itemId);
//     }
//   }
//
//   deleteOne_f(itemId, urlExtra?: UrlExtra) {
//     if (urlExtra) {
//     return this.deleteService_f(itemId + '/' + urlExtra.path);
//     } else {
//       return this.deleteService_f(itemId);
//     }
//   }
// deleteOne_f(itemId, urlExtra?: UrlExtra) {
//   if (urlExtra) {
//     return this.deleteService_f(itemId + '/' + urlExtra.path);
//   } else {
//     return this.deleteService_f(itemId);
//   }
// }
//   getOne(itemId, urlExtra?: UrlExtra, responseContentType?: ResponseContentType): any {
//     if (urlExtra) {
//       if (responseContentType) {
//         return this.getService(itemId + '/' + urlExtra.path, responseContentType);
//       } else {
//         return this.getService(itemId + '/' + urlExtra.path);
//       }
//     } else {
//       if (responseContentType) {
//         return this.getService(itemId, responseContentType);
//       } else {
//         return this.getService(itemId);
//       }
//     }
//
//   }
//
//   getOne_f(itemId, urlExtra?: UrlExtra, responseContentType?: ResponseContentType): any {
//     if (urlExtra) {
//       if (responseContentType) {
//         return this.getService_f(itemId + '/' + urlExtra.path);
//       } else {
//         return this.getService_f(itemId + '/' + urlExtra.path);
//       }
//     } else {
//       if (responseContentType) {
//         return this.getService_f(itemId);
//       } else {
//         return this.getService_f(itemId);
//       }
//     }
//   }
//
//
//   updateOne(item: T, urlExtra?: UrlExtra, itemId?: string) {
//     if (urlExtra) {
//       if (itemId) {
//         return this.putService(itemId + '/' + urlExtra.path, item);
//       } else {
//         return this.putService( urlExtra.path, item);
//       }
//     } else {
//       if (itemId) {
//         return this.putService(itemId, item);
//       } else {
//         return this.putService('', item);
//       }
//     }
//
//   }
//
//   updateOne_f(item: T, urlExtra?: UrlExtra, itemId?: string) {
//     if (urlExtra) {
//       if (itemId) {
//         return this.putService_f(itemId + '/' + urlExtra.path, item);
//       } else {
//         return this.putService_f( urlExtra.path, item);
//       }
//     } else {
//       if (itemId) {
//         return this.putService_f(itemId, item);
//       } else {
//         return this.putService_f('', item);
//       }
//     }
//   }
//

// import {ResponseContentType} from './response-content-type';
// import {RestExtra} from './rest-extra';
//
// // @Injectable()
var HelperServiceBase2 = /** @class */ (function (_super) {
    __extends(HelperServiceBase2, _super);
    function HelperServiceBase2() {
        return _super.call(this) || this;
    }
    HelperServiceBase2.prototype.helperGetService = function (suffixPath, objectSuffix, prefix, restExtra) {
        return _super.prototype.getService.call(this, this.getRestExtra(suffixPath, objectSuffix, prefix, restExtra));
    };
    HelperServiceBase2.prototype.helperDeleteService = function (suffixPath, objectSuffix, prefix, restExtra) {
        return this.deleteService(this.getRestExtra(suffixPath, objectSuffix, prefix, restExtra));
    };
    HelperServiceBase2.prototype.helperPutService = function (value, suffixPath, objectSuffix, prefix, restExtra) {
        return this.putService(value, this.getRestExtra(suffixPath, objectSuffix, prefix, restExtra));
    };
    HelperServiceBase2.prototype.helperPostService = function (value, suffixPath, objectSuffix, prefix, restExtra) {
        return this.postService(value, this.getRestExtra(suffixPath, objectSuffix, prefix, restExtra));
    };
    HelperServiceBase2.prototype.getRestExtra = function (suffixPath, objectSuffix, prefix, restExtra) {
        var myRestExtra;
        if (isNullOrUndefined(restExtra)) {
            myRestExtra = new RestExtra({});
        }
        else {
            myRestExtra = restExtra;
        }
        myRestExtra.objectSuffix = this.replaceParams(suffixPath, objectSuffix);
        myRestExtra.objectPrefix = this.getPrefix(prefix);
        return myRestExtra;
        // return {
        //   objectPrefix: this.getPrefix(prefix),
        //   objectSuffix: this.replaceParams(suffixPath, objectSuffix),
        //   requestContentType: isNullOrUndefined(restExtra) ? RequestContentType.JSON : restExtra.requestContentType,
        //   responseContentType: isNullOrUndefined(restExtra) ? ResponseContentType.Json : restExtra.responseContentType,
        //   urlQuery: isNullOrUndefined(restExtra) ? '' : restExtra.urlQuery,
        //   useObjectPrefix: isNullOrUndefined(restExtra) ? true : restExtra.useObjectPrefix,
        //   useObjectSuffix: isNullOrUndefined(restExtra) ? true : restExtra.useObjectSuffix,
        //   useObjectName: isNullOrUndefined(restExtra) ? true : restExtra.useObjectName
        // };
    };
    return HelperServiceBase2;
}(ServiceBase2));

var AssetsService = /** @class */ (function () {
    function AssetsService(_HttpClient) {
        this._HttpClient = _HttpClient;
    }
    AssetsService.prototype.getData = function (url, extra) {
        var myExtra = extra;
        if (isNullOrUndefined(extra)) {
            myExtra = { responseContentType: ResponseContentType.Json };
        }
        else if (isNullOrUndefined(extra.responseContentType)) {
            myExtra.responseContentType = ResponseContentType.Json;
        }
        return this._HttpClient.get(url, { responseType: ServiceUtil.getResponseContentType(myExtra.responseContentType) })
            .toPromise()
            .then(function (response) {
            return response;
        }).catch(function (err) {
            console.log(err);
            return null;
        });
    };
    AssetsService.ctorParameters = function () { return [
        { type: HttpClient }
    ]; };
    AssetsService.ngInjectableDef = ɵɵdefineInjectable({ factory: function AssetsService_Factory() { return new AssetsService(ɵɵinject(HttpClient)); }, token: AssetsService, providedIn: "root" });
    AssetsService = __decorate([
        Injectable({
            providedIn: 'root'
        })
    ], AssetsService);
    return AssetsService;
}());

function isFunction(value) {
    return typeof value === 'function';
}
var takeUntilDestroyed = function (componentInstance, destroyMethodName) {
    if (destroyMethodName === void 0) { destroyMethodName = 'ngOnDestroy'; }
    return function (source) {
        var originalDestroy = componentInstance[destroyMethodName];
        if (isFunction(originalDestroy) === false) {
            throw new Error(componentInstance.constructor.name + " is using untilDestroyed but doesn't implement " + destroyMethodName);
        }
        if (!componentInstance['__takeUntilDestroy']) {
            componentInstance['__takeUntilDestroy'] = new Subject();
            componentInstance[destroyMethodName] = function () {
                // tslint:disable-next-line:no-unused-expression
                isFunction(originalDestroy) && originalDestroy.apply(this, arguments);
                componentInstance['__takeUntilDestroy'].next(true);
                componentInstance['__takeUntilDestroy'].complete();
            };
        }
        return source.pipe(takeUntil(componentInstance['__takeUntilDestroy']));
    };
};

/*
 * Public API Surface of core
 */

/**
 * Generated bundle index. Do not edit.
 */

export { AssetsService, CacheModule, CacheService, Config, CoreComponent, CoreModule, CoreService, DataService, ErrorHandle, ExtendedServiceBase2, FakeData, FakeServiceBase, FakeServiceBase2, GlobalConfigurations, HandleResponse, HelperServiceBase2, Keyword, RequestContentType, ResponseContent, ResponseContentType, RestExtra, ServiceBase, ServiceBase2, ServiceConfig, ServiceUtil, State, StoredItem, TokenMode, takeUntilDestroyed, CacheLocalStorage as ɵa, CacheStorageAbstract as ɵb, CacheSessionStorage as ɵc, CacheMemoryStorage as ɵd };
//# sourceMappingURL=angular-boot-core.js.map
