import { CacheSessionStorage } from './util/services/storage/session-storage/cache-session-storage.service';
import { CacheLocalStorage } from './util/services/storage/local-storage/cache-local-storage.service';
import { CacheMemoryStorage } from './util/services/storage/memory/cache-memory.service';
import { CacheOptionsInterface } from './util/interfaces/cache-options.interface';
import { CacheType } from './util/enums/cache-type';
import { StoredItem } from './helper/stored-item';
import { Observable } from 'rxjs';
export declare class CacheService {
    private localStorage;
    private sessionStorage;
    private memoryStorage;
    /**
     * Cache prefix
     */
    private _prefix;
    /**
     * Default cache options
     * CacheOptionsInterface
     */
    private _defaultOptions;
    private _defaultStorageType;
    localStorageStoredItemList: StoredItem[];
    sessionStorageStoredItemList: StoredItem[];
    memoryStoredItemList: StoredItem[];
    constructor(localStorage: CacheLocalStorage, sessionStorage: CacheSessionStorage, memoryStorage: CacheMemoryStorage);
    setItem(key: string, value: any, storageType: CacheType, options?: CacheOptionsInterface): void;
    getItem(key: string, storageType: CacheType): Observable<any>;
    existsItem(key: string, storageType: CacheType): Observable<boolean>;
    removeItem(key: string, storageType: CacheType): void;
    private setToStoredList;
    private getRelatedList;
    /**
     * Get data from cache
     * key
     * {any}
     */
    get(key: string, storageType: CacheType): any;
    set(key: string, value: any, storageType: CacheType, options?: CacheOptionsInterface): boolean;
    /**
     * Check if value exists
     * key
     * {boolean}
     */
    exists(key: string, storageType: CacheType): boolean;
    /**
     * Remove item from cache
     * key
     */
    remove(key: string, storageType: CacheType): void;
    /**
     * Remove key from tags keys list
     */
    private _removeFromTag;
    /**
     * Remove all from cache
     */
    removeAll(storageType: CacheType): void;
    /**
     * Get all tag data
     */
    getTagData(tag: string, storageType: CacheType): {
        [key: string]: any;
    };
    /**
     * Remove all by tag
     */
    removeTag(tag: string, storageType: CacheType): void;
    /**
     * Set global cache key prefix
     */
    setGlobalPrefix(prefix: string): void;
    private _toStorageKey;
    private _fromStorageKey;
    private _getCachePrefix;
    private getMyValidatedStorage;
    /**
     * Validate cache storage
     */
    private _validateStorage;
    /**
     * Validate storage value
     */
    private _validateStorageValue;
    /**
     * Prepare value to set to storage
     */
    private _toStorageValue;
    /**
     * Prepare options to set to storage
     */
    private _toStorageOptions;
    /**
     * Save tag to list of tags
     */
    private _saveTag;
    private _tagsStorageKey;
    /**
     * check if its system cache key
     */
    private _isSystemKey;
    private removeFromStoredList;
}
